%{

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#define YYDEBUG 1

#define YYSTYPE TreeNode *

#include "util.h"
#include "global.h"
#include "aldor_parser.tab.h"
#include "lex.yy.c"


static TreeNode * savedTree;

extern int yyparse();


int yyerror(char *s) {
    printf("Syntax Error: %s At Line %d\n", s, line_num);
}

%}


/* RESERVE WORDS  */
%token ADD ALWAYS AND ASSERT BREAK BUT CATCH DEFAULT DEFINE DELAY DO
       ELSE EXCEPT EXPORT EXTEND FIX FLUID FOR FREE FROM GENERATE GOTO
       HAS IF IMPORT IN INLINE IS ISNT ITERATE LET LOCAL MACRO NEVER
       NOT OF OR PRETEND REF REPEAT RETURN RULE SELECT THEN THROW TO TRY
       WHERE WHILE WITH YIELD BY CASE FINALLY MOD QUO REM


/* SYMBOLS - DEFINEABLE */
%token SETMINUS POUND TIMES PLUS MINUS DIVIDE LESSTHAN EQUALS GREATERTHAN
       ATSIGN TILDE CARET VERY_LESS VERY_GREATER LESSEQ GREATEREQ GETS
       TO_SYMBOL JOIN MEET STARSTAR DOTDOT UNEQUALS CARETEQUALS


/* SYMBOLS - RESERVED OPS */
%token DOLLAR SQUOTE LPAREN RPAREN FUN_ARROW COMMA COLON IS_ASSIGNED
       SEMI MACRO_IS EXIT LBRACK RBRACK LBRACE RBRACE VERY_EQUAL BAR


/* SYMBOLS - FUTURE OPS */
%token LLBRACK LLBRACE LLPAREN RRBRACK RRBRACE RRPAREN GRAVE
       AMPERSAND OROR


/* OTHERS */
%token INT ID FLOAT PREDOC POSTDOC STRING
       COLON_STAR DCOLON DOT PLUSMINUS BACKSET BACKTAB SETTAB


%%
start:    curlyContents_Labelled
          {
	    $$ = newBlockNode(StartK);
	    $$->child[0] = $1;
	    $$->start_position = @$.first_column;
	    $$->end_position = @$.last_column;
	    $$->token_type = "start";

	    savedTree = $$;
	  }

expression:   labelled
              {
		$$ = $1;
	      }
           |  expression SEMI labelled
              {
		$$ = newTempNode(TK);
		$$->start_position = @$.first_column;
		$$->end_position = @$.last_column;
		$$->token_type = "exp";
		$$->child[0] = $1;
		$$->child[1] = $3;
	      }
           |  expression SEMI
	      {
		$$ = $1;
	      }
                                         

labelled  : comma 
            {
	      $$ = $1;
	    }
          | declaration
            {
	      $$ = $1;
	    } 
          | ATSIGN atom labelled
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "label";
	      $$->attr.op = ATSIGN;
	      $$->child[0] = $2;
	      $$->child[1] = $3;
	    } 
          | ATSIGN atom 
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "label";
	      $$->attr.op = ATSIGN;
	      $$->child[0] = $2;
	    }


declaration :
            MACRO sig
            {
	      $$ = newDeclNode( MacroK );
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "declaration";
	      $$->child[0] = $2;
	    }
          | EXTEND sig
            {
	      $$ = newDeclNode( ExtendK );
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "declaration";
	      $$->child[0] = $2;
	    }
          | LOCAL sig
            {
	      $$ = newDeclNode( LocalK );
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "declaration";
	      $$->child[0] = $2;
	    }
          | FLUID sig
            {
	      $$ = newDeclNode( FluidK );
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "declaration";
	      $$->child[0] = $2;
	    }
          | DEFAULT sig
            {
	      $$ = newDeclNode( DefaultK );
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "declaration";
	      $$->child[0] = $2;
	    }
          | DEFINE sig
            {
	      $$ = newDeclNode( DefineK );
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "declaration";
	      $$->child[0] = $2;
	    }
          | FIX sig
            {
	      $$ = newDeclNode( FixK );
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "declaration";
	      $$->child[0] = $2;
	    }
          | INLINE sigOp fromPartOpt
            {
	      $$ = newDeclNode( InlineK );
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "declaration";
	      $$->child[0] = $2;
	    }
          | IMPORT sigOp fromPartOpt
            {
	      $$ = newDeclNode( ImportK );
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "declaration";
	      $$->child[0] = $2;
	    }
          | EXPORT sigOp
            {
	      $$ = newDeclNode( ExportK );
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "declaration";
	      $$->child[0] = $2;
	    }
          | EXPORT sigOp toPart
            {
	      $$ = newDeclNode( ExportK );
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "declaration";
	      $$->child[0] = $2;
	      $$->child[1] = $3;
	    }
          | EXPORT sigOp fromPart
            {
	      $$ = newDeclNode( ExportK );
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "declaration";
	      $$->child[0] = $2;
	      $$->child[1] = $3;
	    }

toPart:
            TO infixed               
            {
	      $$ = newStmtNode(ToK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "toPart";
	      $$->child[0] = $2;
	    }
fromPartOpt:                             
            fromPart
            {
	      $$ = $1;
	    }        
          | /* empty */
            {
	      $$ = NULL;   //Tree = NULL;
	    }

fromPart:
            FROM infixedCL
            {
	      $$ = newExpNode( OpK );
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "fromPart";
	      $$->attr.op = FROM;
	      $$->child[0] = $2;
	    }

infixedCL:
            infixed
            {
	      $$ = $1;
	    }
          | infixedCL COMMA infixed
            {
	      $$ = newTempNode(TK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "infixedCL";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

sigOp:
            sig
            {
	      $$ = $1;
	    }
          | /* empty */
            {
	      $$ = NULL;   //Tree = NULL;
	    }

sig:
            declBinding
            {
	      $$ = $1;
	    }
          | block
            {
	      $$ = $1;
	    }

declPart:
            COLON type
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "declPart";
	      $$->attr.op = COLON;
	      $$->child[0] = $2;
	    }                 
          | COLON_STAR type
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "declPart";
	      $$->attr.op = COLON_STAR;
	      $$->child[0] = $2;
	    }

comma:
	    commaItem
            {
	      $$ = $1;
	    }
          | commaItem COMMA comma
            {
		$$ = newTempNode(TK);
		$$->start_position = @$.first_column;
		$$->end_position = @$.last_column;
		$$->token_type = "comma";
		$$->child[0] = $1;
		$$->child[1] = $3;
	    }       

commaItem:
            binding_AS
            {
	      $$ = $1;
	    }        
          | binding_AS WHERE commaItem
            {
	      $$ = newStmtNode( WhereK );
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "commaItem";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

declBinding:
            bindingR_IED_AS
            {
	      $$ = $1;
	    }        

infixedExprsDecl:
            infixedExprs
            {
	      $$ = $1;
	    }         
          | infixedExprs declPart       
            {
	      $$ = $1;
	      $$->child[0] = $2;
	    }


infixedExprs:
	    infixedExpr                 
            {
	      $$ = $1;
	    }
          | infixedExpr COMMA infixedExprs
            {
	      $$ = newTempNode(TK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "infixedExprs";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

dgc_Binding_Symbol:
            IS_ASSIGNED
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "dgc_Binding_Symbol";
	      $$->attr.op = IS_ASSIGNED;
	    }        
          | VERY_EQUAL
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "dgc_Binding_Symbol";
	      $$->attr.op = VERY_EQUAL;
	    }  
          | MACRO_IS                    
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "dgc_Binding_Symbol";
	      $$->attr.op = MACRO_IS;
	    }  
          | FUN_ARROW                   
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "dgc_Binding_Symbol";
	      $$->attr.op = FUN_ARROW;
	    }  


bindingR_IED_AS:
	    infixedExprsDecl
            {
	      $$ = $1;
	    }
          | infixedExprsDecl dgc_Binding_Symbol binding_AS
            {
	      $$ = $2;
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "bindingR_IED_AS";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

binding_AS:
            bindingL_Infixed_AS 
            {
	      $$ = $1;
	    }
        

binding_BS:
            bindingL_Infixed_BS         
            {
	      $$ = $1;
	    }

binding_Collection:
            bindingL_Infixed_Collection 
            {
	      $$ = $1;
	    }

bindingL_Infixed_AS:
            anyStatement
            {
	      $$ = $1;
	    }                
          | infixed dgc_Binding_Symbol binding_AS
            {
	      $$ = $2;
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "bindingL_Infixed_AS";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

bindingL_Infixed_Collection:
            collection
            {
	      $$ = $1;
	    }        
          | infixed dgc_Binding_Symbol binding_Collection
            {
	      $$ = $2;
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "bindingL_Infixed_Collection";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

bindingL_Infixed_BS:
            balStatement
            {
	      $$ = $1;
	    }        
          | infixed dgc_Binding_Symbol binding_BS
            {
	      $$ = $2;
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "bindingL_Infixed_BS";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

anyStatement:
            IF commaItem THEN binding_AS
            {
	      $$ = newStmtNode(IfK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	    $$->token_type = "anyStatement";
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	    }
          | flow_as                     
            {
	      $$ = $1;
	    }

balStatement:
            flow_bs                  
            {
	      $$ = $1;
	    }

flow_as:  
            collection
            {
	      $$ = $1;
	    }
          | IF commaItem
            THEN binding_BS
            ELSE binding_AS
            {
	      $$ = newStmtNode(IfK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	      $$->child[2] = $6;
	    }
          | collection EXIT binding_AS
            {
	      $$ = newStmtNode( ExitK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | iterators REPEAT binding_AS
            {
	      $$ = newStmtNode(RepeatK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | TRY binding_AS CATCH casesOpt finalPart_AS
            {
	      $$ = newStmtNode(TryK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	      $$->child[2] = $5;
	    }
          | TRY binding_AS CATCH casesOpt alwaysPart_AS
            {
	      $$ = newStmtNode(TryK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	      $$->child[2] = $5;
	    }
          | SELECT binding_AS IN cases    
            {
	      $$ = newStmtNode(SelectK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	    }
          | DO binding_AS                    
            {
	      $$ = newStmtNode(DoK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	      $$->child[0] = $2;
	    }
          | DELAY binding_AS                 
            {
	      $$ = newStmtNode(DelayK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	      $$->child[0] = $2;
	    }
          | GENERATE genBound binding_AS     
            {
	      $$ = newStmtNode(GenerateK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	      $$->child[0] = $2;
	      $$->child[1] = $3;
	    }
          | ASSERT binding_AS                
            {
	      $$ = newStmtNode(AssertK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	      $$->child[0] = $2;
	    }
          | ITERATE                          
            {
	      $$ = newStmtNode(IterateK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	    }
          | ITERATE name                     
            {
	      $$ = newStmtNode(IterateK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	      $$->child[0] = $2;
	    }
          | BREAK                            
            {
	      $$ = newStmtNode(BreakK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	    }
          | BREAK name                       
            {
	      $$ = newStmtNode(BreakK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	      $$->child[0] = $2;
	    }
          | RETURN
            {
	      $$ = newStmtNode(ReturnK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	    }
          | RETURN collection                
            {
	      $$ = newStmtNode(ReturnK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	      $$->child[0] = $2;
	    }
          | YIELD binding_AS                 
            {
	      $$ = newStmtNode(YieldK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	      $$->child[0] = $2;
	    }
          | EXCEPT binding_AS                
            {
	      $$ = newStmtNode(ExceptK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	      $$->child[0] = $2;
	    }
          | GOTO id                          
            {
	      $$ = newStmtNode(GotoK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	      $$->child[0] = $2;
	    }
          | NEVER
            {
	      $$ = newStmtNode(NeverK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_as";
	    }

flow_bs   : collection                  
            {
	      $$ = $1;
	    }
          | IF commaItem
            THEN binding_BS
            ELSE binding_BS
            {
	      $$ = newStmtNode(IfK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	      $$->child[2] = $6;
	    }
          | collection EXIT binding_BS  
            {
	      $$ = newStmtNode(ExitK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | iterators REPEAT binding_BS
            {
	      $$ = newStmtNode(RepeatK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | TRY binding_AS CATCH casesOpt finalPart_BS
            {
	      $$ = newStmtNode(TryK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	      $$->child[2] = $5;
	    }
          | TRY binding_AS CATCH casesOpt alwaysPart_BS
            {
	      $$ = newStmtNode(TryK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	      $$->child[2] = $5;
	    }
          | SELECT binding_AS IN cases    
            {
	      $$ = newStmtNode(SelectK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	    }
          | DO binding_BS                    
            {
	      $$ = newStmtNode(DoK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	      $$->child[0] = $1;
	    }
          | DELAY binding_BS                 
            {
	      $$ = newStmtNode(DelayK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	      $$->child[0] = $2;
	    }
          | GENERATE genBound binding_BS     
            {
	      $$ = newStmtNode(GenerateK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	      $$->child[0] = $2;
	      $$->child[1] = $3;
	    }
          | ASSERT binding_BS                
            {
	      $$ = newStmtNode(AssertK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	      $$->child[0] = $2;
	    }
          | ITERATE
            {
	      $$ = newStmtNode(IterateK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	    }
          | ITERATE name                     
            {
	      $$ = newStmtNode(IterateK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	      $$->child[0] = $2;
	    }
          | BREAK
            {
	      $$ = newStmtNode(BreakK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	    }
          | BREAK name                       
            {
	      $$ = newStmtNode(BreakK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	      $$->child[0] = $2;
	    }
          | RETURN                           
            {
	      $$ = newStmtNode(ReturnK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	    }
          | RETURN collection                
            {
	      $$ = newStmtNode(ReturnK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	      $$->child[0] = $2;
	    }
          | YIELD binding_BS                 
            {
	      $$ = newStmtNode(YieldK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	      $$->child[0] = $2;
	    }
          | EXCEPT binding_BS               
            {
	      $$ = newStmtNode(ExceptK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	      $$->child[0] = $2;
	    }
          | GOTO id                          
            {
	      $$ = newStmtNode(GotoK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	      $$->child[0] = $2;
	    }
          | NEVER
            {
	      $$ = newStmtNode(NeverK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "flow_bs";
	    }

genBound: 
            TO commaItem OF
            {	      
	      $$ = newStmtNode(ToK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "genBound";
	      $$->child[0] = $2;
	    }
	  | /* empty */
            {
	      $$ = NULL;   //Tree = NULL;
	    }

cases:
            binding_Collection
            {
	      $$ = $1;
	    }          

casesOpt: 
            binding_Collection
            {
	      $$ = $1;
	    }                  
	  | /* empty */
            {
	      $$ = NULL;   //Tree = NULL;
	    }

alwaysPart_AS:
	    ALWAYS binding_AS
            {
	      $$ = newStmtNode(AlwaysK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "alwaysPart_AS";
	      $$->child[0] = $1;
	    }

alwaysPart_BS:
            ALWAYS binding_BS
            {
	      $$ = newStmtNode(AlwaysK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "alwaysPart_BS";
	      $$->child[0] = $1;
	    }

finalPart_BS:
            FINALLY binding_BS
            {
	      $$ = newStmtNode(FinallyK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "finalPart_BS";
	      $$->child[0] = $2;
	    }
          | /* empty */
            {
	      $$ = NULL;   //Tree = NULL;
	    }

finalPart_AS:
            FINALLY binding_AS 
            {
	      $$ = newStmtNode(FinallyK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "finalPart_AS";
	      $$->child[0] = $2;
	    }
          | /* empty */
            {
	      $$ = NULL;   //Tree = NULL;
	    }


collection:
            infixed
            {
	      $$ = $1;
	    }
          | infixed iterators           
            {
	      $$ = newTempNode(TK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "collection";
	      $$->child[0] = $1;
	      $$->child[1] = $2;
	    }

iterators:
            iterators1
            {
	      $$ = $1;
	    }
        
iterators1:
            iterator  
            {
	      $$ = $1;
	    }                  
          | iterators1 iterator         
            {
	      $$ = newTempNode(TK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "iterators1";
	      $$->child[0] = $1;
	      $$->child[1] = $2;
	    }

iterator:
	    FOR forLHS IN infixed
            {
	      $$ = newStmtNode(ForK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "iterator";
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	    } 
          | FOR forLHS IN infixed suchThatPart
            {
	      $$ = newStmtNode(ForK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "iterator";
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	      $$->child[2] = $5;
	    }
          | WHILE infixed
            {
	      $$ = newStmtNode(WhileK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "iterator";
	      $$->child[0] = $2;
	    }

forLHS:
            infixed
            {
	      $$ = $1;
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "forLHS";
	    }
          | FREE infixed
            {
	      $$ = newDeclNode( FreeK );
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "forLHS";
	      $$->child[0] = $2;
	    }             
          | LOCAL infixed
            {
	      $$ = newDeclNode( LocalK );
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "forLHS";
	      $$->child[0] = $2;
	    }
          | FLUID infixed
            {
	      $$ = newDeclNode( FluidK );
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "forLHS";
	      $$->child[0] = $2;
	    }

suchThatPart:
            BAR infixed
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "suchThatPart";
	      $$->attr.op = BAR;
	      $$->child[0] = $2; 
	    }

infixed:
	    infixedExpr
            {
	      $$ = $1;
	    }
          | infixedExpr declPart
            {
	      $$ = newTempNode(TK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "infixed";
	      $$->child[0] = $1;
	      $$->child[1] = $2;
	    }        
          | block
            {
	      $$ = $1;
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "infixed";
	    }        

infixedExpr:
            e11_OP
            {
	      $$ = $1;
	    }
          | e3
            {
	      $$ = $1;
	    }

e3:
            e4
            {
	      $$ = $1;
	    }                  
          | e3 AND e4 
            {
	      $$ = newStmtNode(AndK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e3";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }               
          | e3 OR e4
            {
	      $$ = newStmtNode(OrK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e3";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }    
          | e3 latticeOp e4
            {
	      $$ = $2;
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e3";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }


e4:
            e5                          
            {
	      $$ = $1;
	    }
          | e4 HAS e5                
            {
	      $$ = newStmtNode(HasK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e4";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | e4 relationOp e5            
            {
	      $$ = $2;
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e4";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | relationOp e5               
            {
	      $$ = $1;
	      $$->child[0] = $2;
	    }

e5:
            e6                          
            {
	      $$ = $1;
	    }
          | e5 seqOp                    
            {
	      $$ = $2;
	      $$->child[0] = $1;
	    }
          | e5 seqOp e6
            {
	      $$ = $2;
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e5";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

e6:
            e7                          
            {
	      $$ = $1;
	    }
          | e6 plusOp e7                
            {
	      $$ = $2;
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e6";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | plusOp e7                   
            {
	      $$ = $1;
	      $$->child[0] = $2;
	    }
e7:
            e8
            {
	      $$ = $1;
	    }
          | e7 quotientOp e8            
            {
	      $$ = $2;
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e7";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

e8:
            e9
	    {
	      $$ = $1;
	    }
          | e8 timesOp e9               
            {
	      $$ = $2;
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e8";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

e9:         
            e11_e12
            {
	      $$ = $1;
	    }
          | e11_e12 powerOp e9          
            {
	      $$ = $2;
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e9";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

e11_OP:
            op                          
            {
	      $$ = $1;
	    }
          | e11_OP DCOLON e12           
	    {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e11_OP";
	      $$->attr.op = DCOLON;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | e11_OP ATSIGN e12           
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e11_OP";
	      $$->attr.op = ATSIGN;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | e11_OP PRETEND e12       
            {
	      $$ = newStmtNode(PretendK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e11_OP";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

e11_e12:
            e12                         
            {
	      $$ = $1;
	    }
          | e11_e12 DCOLON e12          
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e11_e12";
	      $$->attr.op = DCOLON;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | e11_e12 ATSIGN e12          
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e11_e12";
	      $$->attr.op = ATSIGN;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | e11_e12 PRETEND e12      
            {
	      $$ = newStmtNode(PretendK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e11_e12";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }


type:
            e11_e12
            {
	      $$ = $1;
	    }

e12:
            e13
            {
	      $$ = $1;
	    }
          | e13 arrowOp e12             
            {
	      $$ = $2;
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e12";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

e13:
            e14
            {
	      $$ = $1;
	    }
          | e14 DOLLAR qualTail
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e13";
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

qualTail: 
            leftJuxtaposed
            {
	      $$ = $1;
	    }
          | leftJuxtaposed DOLLAR qualTail
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "qualTail";
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
                      
opQualTail:
	    molecule                    
            {
	      $$ = $1;
	    }
          | molecule DOLLAR opQualTail
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "opQualTail";
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

e14:
            e15
            {
	      $$ = $1;
	    }
          | e14 EXCEPT e15           
            {
	      $$ = newStmtNode(ExceptK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e14";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | WITH declMolecule        
            {
	      $$ = newStmtNode(WithK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e14";
	      $$->child[0] = $2;
	    }
          | e14 WITH declMolecule    
            {
	      $$ = newStmtNode(WithK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e14";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | ADD declMolecule         
            {
	      $$ = newStmtNode(AddK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e14";
	      $$->child[0] = $2;
	    }
          | e14 ADD declMolecule     
            {
	      $$ = newStmtNode(AddK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "e14";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

e15:
            application                 
            {
	      $$ = $1;
	    }
op:
            arrowOp
            {
	      $$ = $1;
	    }
          | latticeOp                  
            {
	      $$ = $1;
	    }
          | relationOp                  
            {
	      $$ = $1;
	    }
          | seqOp                       
            {
	      $$ = $1;
	    }
          | plusOp                      
            {
	      $$ = $1;
	    }
          | quotientOp                  
            {
	      $$ = $1;
	    }
          | timesOp                     
            {
	      $$ = $1;
	    }
          | powerOp                     
            {
	      $$ = $1;
	    }

nakedOp:
            arrowTok                    
            {
	      $$ = $1;
	    }
          | latticeTok                  
            {
	      $$ = $1;
	    }
          | relationTok                 
            {
	      $$ = $1;
	    }
          | seqTok                      
            {
	      $$ = $1;
	    }
          | plusTok                     
            {
	      $$ = $1;
	    }
          | quotientTok                 
            {
	      $$ = $1;
	    }
          | timesTok                    
            {
	      $$ = $1;
	    }
          | powerTok                    
            {
	      $$ = $1;
	    }

arrowOp:
            arrowTok                    
            {
	      $$ = $1;
	    }
          | arrowTok DOLLAR opQualTail  
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "arrowOp";
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
latticeOp:
            latticeTok                  
            {
	      $$ = $1;
	    }
          | latticeTok DOLLAR opQualTail 
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "latticeOp";
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

relationOp:
            relationTok                 
            {
	      $$ = $1;
	    }
          | relationTok DOLLAR opQualTail 
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "relationOp";
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

seqOp:
            seqTok                      
            {
	      $$ = $1;
	    }
          | seqTok DOLLAR opQualTail    
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "seqOp";
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

plusOp:
            plusTok                     
            {
	      $$ = $1;
	    }
          | plusTok DOLLAR opQualTail   
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "plusOp";
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

quotientOp:
            quotientTok               
            {
	      $$ = $1;
	    }
          | quotientTok DOLLAR opQualTail 
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "quotientOp";
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

timesOp:
            timesTok                    
            {
	      $$ = $1;
	    }
          | timesTok DOLLAR opQualTail  
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "timesOp";
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
powerOp:
            powerTok                    
            {
	      $$ = $1;
	    }
          | powerTok DOLLAR opQualTail  
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "powerOp";
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

arrowTok:
            GETS                        
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "arrowTok";
	      $$->attr.op = GETS;
	    }
          | TO_SYMBOL                   
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "arrowTok";
	      $$->attr.op = TO_SYMBOL;
	    }

latticeTok:
            JOIN                        
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "latticeTok";
	      $$->attr.op = JOIN;
	    }
          | MEET                        
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "latticeTok";
	      $$->attr.op = MEET;
	    }

relationTok:
            ISNT
            {
	      $$ = newStmtNode(IsntK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "relationTok";
	    }
          | IS
            {
	      $$ = newStmtNode(IsK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "relationTok";
	    }
          | CASE
            {
	      $$ = newStmtNode(CaseK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "relationTok";
	    }
          | VERY_LESS                   
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "relationTok";
	      $$->attr.op = VERY_LESS;
	    }
          | VERY_GREATER                
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "relationTok";
	      $$->attr.op = VERY_GREATER;
	    }
          | LESSTHAN                        
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "relationTok";
	      $$->attr.op = LESSTHAN;
	    }
          | LESSEQ                      
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "relationTok";
	      $$->attr.op = LESSEQ;
	    }
          | EQUALS                      
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "relationTok";
	      $$->attr.op = EQUALS;
	    }
          | GREATEREQ                   
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "relationTok";
	      $$->attr.op = GREATEREQ;
	    }
          | GREATERTHAN
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "relationTok";
	      $$->attr.op = GREATERTHAN;
	    }
	  | UNEQUALS
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "relationTok";
	      $$->attr.op = UNEQUALS;
	    }
          | CARETEQUALS
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "relationTok";
	      $$->attr.op = CARETEQUALS;
	    }

seqTok:
            BY
            {
	      $$ = newStmtNode(ByK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "seqTok";
	    }
          | DOTDOT                      
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "seqTok";
	      $$->attr.op = DOTDOT;
	    }

plusTok:
            PLUS                        
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "plusTok";
	      $$->attr.op = PLUS;
	    }
          | MINUS                       
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "plusTok";
	      $$->attr.op = MINUS;
	    }
          | PLUSMINUS                   
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "plusTok";
	      $$->attr.op = PLUSMINUS;
	    }

quotientTok:
            MOD                         
            {
	      $$ = newStmtNode(ModK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	    $$->token_type = "quotientTok";
	    }
          | QUO                         
            {
	      $$ = newStmtNode(QuoK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "quotientTok";
	    }
          | REM                         
            {
	      $$ = newStmtNode(RemK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	    $$->token_type = "quotientTok";
	    }

timesTok:
            TIMES                       
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "timesTok";
	      $$->attr.op = TIMES;
	    }
          | DIVIDE                      
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "timesTok";
	      $$->attr.op = DIVIDE;
	    }
          | SETMINUS                    
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "timesTok";
	      $$->attr.op = SETMINUS;
	    }

powerTok:
            CARET                       
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "powerTok";
	      $$->attr.op = CARET;
	    }
          | STARSTAR                    
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "powerTok";
	      $$->attr.op = STARSTAR;
	    }

application:
            rightJuxtaposed
            {
	      $$ = $1;
	    }

rightJuxtaposed:
            jRight_Mol            
            {
	      $$ = $1;
	    }

leftJuxtaposed:
            jLeft_Mol             
            {
	      $$ = $1;
	    }

jRight_Mol:
            jLeft_Mol                   
            {
	      $$ = $1;
	    }
          | jLeft_Mol jRight_Atom       
            {
	      $$ = newTempNode(TK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "jRight_Mol";
	      $$->child[0] = $1;
	      $$->child[1] = $2;
	    }
          | NOT jRight_Atom
            {
	      $$ = newStmtNode(NotK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "jRight_Mol";
	      $$->child[0] = $2;
	    }


jRight_Atom:
            jLeft_Atom                 
            {
	      $$ = $1;
	    }
          | jLeft_Atom jRight_Atom      
            {
	      $$ = newTempNode(TK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "jRight_Atom";
	      $$->child[0] = $1;
	      $$->child[1] = $2;
	    }
          | NOT jRight_Atom          
            {
	      $$ = newStmtNode(NotK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "jRight_Atom";
	      $$->child[0] = $2;
	    }

jLeft_Mol :
            molecule                    
            {
	      $$ = $1;
	    }
          | NOT blockEnclosure       
            {
	      $$ = newStmtNode(NotK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "jLeft_Mol";
	      $$->child[0] = $2;
	    }
          | jLeft_Mol blockEnclosure    
            {
	      $$ = newTempNode(TK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "jLeft_Mol";
	      $$->child[0] = $1;
	      $$->child[1] = $2;
	    }
          | jLeft_Mol DOT blockMolecule     
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "jLeft_Mol";
	      $$->attr.op = DOT;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

jLeft_Atom:
            atom
            {
	      $$ = $1;
	    }
          | NOT blockEnclosure       
            {
	      $$ = newStmtNode(NotK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	    $$->token_type = "jLeft_Atom";
	      $$->child[0] = $2;
	    }
          | jLeft_Atom blockEnclosure   
            {
	      $$ = newTempNode(TK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "jLeft_Atom";
	      $$->child[0] = $1;
	      $$->child[1] = $2;
	    }
          | jLeft_Atom DOT blockMolecule    
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "jLeft_Atom";
	      $$->attr.op = DOT;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

molecule  :
            atom                        
            {
	      $$ = $1;
	    }
          | enclosure                   
            {
	      $$ = $1;
	    }

enclosure :
            parened                     
            {
	      $$ = $1;
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "enclosure";
	    }
          | bracketed                   
            {
	      $$ = $1;
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "enclosure";
	    }
          | quotedIds                   
            {
	      $$ = $1;
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "enclosure";
	    }

declMolecule:
            application                 
            {
	      $$ = $1;
	    }
          | block                       
            {
	      $$ = $1;
	    }
          | /* empty */
            {
	      $$ = NULL;   //Tree = NULL;
	    }

blockMolecule:
            atom                        
            {
	      $$ = $1;
	    }
          | enclosure                   
            {
	      $$ = $1;
	    }
          | block                       
            {
	      $$ = $1;
	    }

blockEnclosure:
            enclosure                   
            {
	      $$ = $1;
	    }
          | block                       
            {
	      $$ = $1;
	    }

block:
            piled_Expression            
            {
	      $$ = $1;
	    }
          | curly_Labelled            
            {
	      $$ = $1;
	    }

parened:
            LPAREN RPAREN               
            {
	      $$ = newEncNode(ParenedK);
	    }
          | LPAREN expression RPAREN    
            {
	      $$ = newEncNode(ParenedK);
	      $$->child[0] = $2;
	    }

bracketed:
            LBRACK RBRACK               
            {
	      $$ = newEncNode(BracketedK);
	    }
          | LBRACK expression RBRACK
            {
	      $$ = newEncNode(BracketedK);
	      $$->child[0] = $2;
	    }

quotedIds:
            SQUOTE SQUOTE               
            {
	     $$ = newEncNode(QuotedIdsK);
	    }
          | SQUOTE names SQUOTE
            {	      
	      $$ = newEncNode(QuotedIdsK);
	      $$->child[0] = $2;
	    }

names:
            name                        
            {
	      $$ = $1;
	    }
          | name COMMA names
            {
	      $$ = newTempNode(TK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "names";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }         

atom:  
            id                          
            {
	      $$ = $1;
	    }
          | literal                     
            {
	      $$ = $1;
	    }

id:
            ID 
            {
	      $$ = newExpNode(IdK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "id";
	      $$->attr.name = copyString(tokenString);
	    }
          | POUND
            {	      
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "id";
	      $$->attr.op = POUND;
	    }
          | TILDE
            {
	      $$ = newExpNode(OpK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "id";
	      $$->attr.op = TILDE;
	    }

name:
            ID
            {
	      $$ = newExpNode(IdK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "name";
	      $$->attr.name = copyString(tokenString);
	    }
          | nakedOp
            {
	      $$ = $1;
	    }

literal:  
              INT              
              {
		$$ = newExpNode(ConstK);
		$$->start_position = @$.first_column;
		$$->end_position = @$.last_column;
		$$->token_type = "literal";
		$$->attr.name = copyString(tokenString);
	      }
          |   FLOAT                
              { 
		$$ = newExpNode(ConstK);
		$$->start_position = @$.first_column;
		$$->end_position = @$.last_column;
		$$->token_type = "literal";
		$$->attr.name = copyString(tokenString);
	      }
          |   STRING               
              {
		$$ = newExpNode(ConstK);
		$$->start_position = @$.first_column;
		$$->end_position = @$.last_column;
		$$->token_type = "literal";
		$$->attr.name = copyString(tokenString);
	      }

doc_Expression:
            preDocument expression postDocument
            {
	      $$ = newBlockNode(DocBlockK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "literal";
	      $$->child[0] = $1;
	      $$->child[1] = $2;
	      $$->child[2] = $3;
	    }                                        

preDocument:
            preDocumentList             
            {
	      $$ = $1;
	    }

preDocumentList:
            PREDOC {$$ = newName(copyString(tokenString));} 
            preDocumentList
            {
	      $$ = newExpNode(DocK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "preDocumentList";
	      $$->attr.name = $2->attr.name;
	      $$->sibling = $3;
	      free($2)
	    }
	  | /* empty */
            {
	      $$ = NULL;   //Tree = NULL;
       	    }

postDocument:
            postDocumentList            
            {
	      $$ = $1;
	    }

postDocumentList:
            POSTDOC {$$ = newName(copyString(tokenString));} 
            postDocumentList
            {
	      $$ = newExpNode(DocK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "postDocumentList";
	      $$->attr.name = $2->attr.name;
	      $$->sibling = $3;
	      free($2);
	    }
          | /* empty */
            {
	      $$ = NULL;   //Tree = NULL;
	    }

piled_Expression:
            SETTAB pileContents_Expression BACKTAB
            {
	      $$ = newBlockNode(PiledK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "piled_Expression";
	      $$->child[0] = $2;
	    }

pileContents_Expression:
            doc_Expression              
            {
	      $$ = $1;
	    }
          | pileContents_Expression BACKSET doc_Expression
            {
	      $$ = newTempNode(TK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "pileContents_Expression";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

curly_Labelled:
            LBRACE
            curlyContents_Labelled
            RBRACE
            {
	      $$ = $2;
	    }
                                             
curlyContents_Labelled:
            curlyContentsList_Labelled
            {
	      $$ = newBlockNode(CurlyK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "curlyContents_Labelled";
	      $$->child[0] = $1;
	    }  

curlyContentsList_Labelled:
            curlyContent1_Labelled
            {
	      $$ = $1;
	    }      
          | curlyContent1_Labelled curlyContentB_Labelled %prec SEMI
            {
	      if( $1 == NULL )
		$$ = $2;

	      else
	      {
		$$ = newTempNode(TK);
		$$->start_position = @$.first_column;
		$$->end_position = @$.last_column;
		$$->token_type = "curlyContentsList";
		$$->child[0] = $1;
		$$->child[1] = $2;
	      }
	    }

curlyContent1_Labelled:
            curlyContent1_Labelled curlyContentA_Labelled
            {
	      if( $1 == NULL )
		$$ = $2;
	  
	      else if( $2 == NULL )
		$$ = $1;

	      else
	      {
		$$ = newTempNode(TK);
		$$->start_position = @$.first_column;
		$$->end_position = @$.last_column;
		$$->token_type = "curlyContent1_Labelled";
		$$->child[0] = $1;
		$$->child[1] = $2;
	      }
	    }
	  | /* empty */
            {
	      $$ = NULL;   //Tree = NULL;
	    }

curlyContentA_Labelled:
            curlyContentB_Labelled SEMI postDocument
            {
	      $$ = newTempNode(TK);
	      $$->start_position = @$.first_column;
	      $$->end_position = @$.last_column;
	      $$->token_type = "curlyContentA_Labelled";
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
                                        
                                         
curlyContentB_Labelled:
            preDocument labelled postDocument
            {
	      if( $1 == NULL && $2 == NULL )
		$$ = $3;

	      else if ($1 == NULL && $3 == NULL)
		$$ = $2;

	      else if ($2 == NULL && $3 == NULL)
		$$ = $1;
	      
	      else
	      {
		$$ = newTempNode(TK);
		$$->start_position = @$.first_column;
		$$->end_position = @$.last_column;
		$$->token_type = "curlyContentB_Labelled";
		$$->child[0] = $1;
		$$->child[1] = $2;
		$$->child[2] = $3;
	      }
	    }



%%

#include <string.h>


int main(int argc, char * argv[] )
{

    char* programfile; //the sourse file to compile in pascal


    // check to see if the user input 2 argument one the program and one the
    // file to compiled
    if(argc != 2)
    {
        // log if the format given is not corredt and exits
        printf("two arguments are needed usage: %s <filename.as>\n",argv[0]);
        exit(1);
    }

    programfile = argv[1];

    // opnes the source for use in the compiler then checks to see where the
    // file exist or not if does not log problem and exits
    yyin = fopen(programfile, "r");
    if(yyin == NULL)
    {
        printf("compile file not found logout\n");
        exit(1);
    }

    yyparse();

    printTree(savedTree, programfile);

    fclose(yyin);
    return 0;
}



