%{

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#define YYDEBUG 1

#define YYSTYPE TreeNode *

#include "util.h"
#include "global.h"
#include "parser.tab.h"
#include "lex.yy.c"


static TreeNode * savedTree;

extern int yyparse();


int yyerror(char *s) {
    printf("Syntax Error: %s At Line %d\n", s, line_num);
}

%}


/* RESERVE WORDS  */
%token ADD ALWAYS AND ASSERT BREAK BUT CATCH DEFAULT DEFINE DELAY DO
       ELSE EXCEPT EXPORT EXTEND FIX FLUID FOR FREE FROM GENERATE GOTO
       HAS IF IMPORT IN INLINE IS ISNT ITERATE LET LOCAL MACRO NEVER
       NOT OF OR PRETEND REF REPEAT RETURN RULE SELECT THEN THROW TO TRY
       WHERE WHILE WITH YIELD BY CASE FINALLY MOD QUO REM


/* SYMBOLS - DEFINEABLE */
%token SETMINUS POUND TIMES PLUS MINUS DIVIDE LESSTHAN EQUALS GREATERTHAN
       ATSIGN TILDE CARET VERY_LESS VERY_GREATER LESSEQ GREATEREQ GETS
       TO_SYMBOL JOIN MEET STARSTAR DOTDOT UNEQUALS CARETEQUALS


/* SYMBOLS - RESERVED OPS */
%token DOLLAR SQUOTE LPAREN RPAREN FUN_ARROW COMMA COLON IS_ASSIGNED
       SEMI MACRO_IS EXIT LBRACK RBRACK LBRACE RBRACE VERY_EQUAL BAR


/* SYMBOLS - FUTURE OPS */
%token LLBRACK LLBRACE LLPAREN RRBRACK RRBRACE RRPAREN GRAVE
       AMPERSAND OROR


/* OTHERS */
%token INT ID FLOAT PREDOC POSTDOC STRING
       COLON_STAR DCOLON DOT PLUSMINUS BACKSET BACKTAB SETTAB


%%
start:    curlyContents_Labelled
          {
	    // printf("start\n");
	    print_overlay( @$.first_column, @$.last_column, "start");

	    $$ = newBlockNode(StartK);
	    $$->child[0] = $1;
	    savedTree = $$;
	  }

expression:   labelled
              {
		// printf("exp1\n");
		print_overlay( @$.first_column, @$.last_column, "exp");
		$$ = $1;
	      }
           |  expression SEMI labelled
              {
		print_overlay( @$.first_column, @$.last_column, "exp");

		$$ = newTempNode(TK);
		$$->child[0] = $1;
		$$->child[1] = $3;
	      }
           |  expression SEMI
	      {
		// printf("exp3\n");
		print_overlay( @$.first_column, @$.last_column, "exp");

		$$ = $1;
	      }
                                         

labelled  : comma 
            {
	      // printf("lab\n");
	      print_overlay( @$.first_column, @$.last_column, "label");

	      $$ = $1;

	      //printTree($$);
	    }
          | declaration
            {
	      // printf("lab2\n");
	      print_overlay( @$.first_column, @$.last_column, "label");
	      
	      $$ = $1;
	    } 
          | ATSIGN atom labelled
            {
	      // printf("lab3\n");
	      print_overlay( @$.first_column, @$.last_column, "label");

	      $$ = newExpNode(OpK);
	      $$->attr.op = ATSIGN;
	      $$->child[0] = $2;
	      $$->child[1] = $3;
	    } 
          | ATSIGN atom 
            {
	      // printf("lab4\n");
	      print_overlay( @$.first_column, @$.last_column, "label");

	      $$ = newExpNode(OpK);
	      $$->attr.op = ATSIGN;
	      $$->child[0] = $2;
	    }


declaration :
            MACRO sig
            {
	      // printf("decl1\n");
	      print_overlay( @$.first_column, @$.last_column, "declaration");

	      $$ = newDeclNode( MacroK );
	      $$->child[0] = $2;
	    }
          | EXTEND sig
            {
	      // printf("decl2\n");
	      print_overlay( @$.first_column, @$.last_column, "declaration");

	      $$ = newDeclNode( ExtendK );
	      $$->child[0] = $2;
	    }
          | LOCAL sig
            {
	      // printf("decl3\n");
	      print_overlay( @$.first_column, @$.last_column, "declaration");
	      
	      $$ = newDeclNode( LocalK );
	      $$->child[0] = $2;
	    }
          | FLUID sig
            {
	      // printf("decl4\n");
	      print_overlay( @$.first_column, @$.last_column, "declaration");
	      
	      $$ = newDeclNode( FluidK );
	      $$->child[0] = $2;
	    }
          | DEFAULT sig
            {
	      // printf("decl5\n");
	      print_overlay( @$.first_column, @$.last_column, "declaration");
	      $$ = newDeclNode( DefaultK );
	      $$->child[0] = $2;
	    }
          | DEFINE sig
            {
	      // printf("decl6\n");
	      print_overlay( @$.first_column, @$.last_column, "declaration");
	      $$ = newDeclNode( DefineK );
	      $$->child[0] = $2;
	    }
          | FIX sig
            {
	      // printf("decl7\n");
	      print_overlay( @$.first_column, @$.last_column, "declaration");
	      $$ = newDeclNode( FixK );
	      $$->child[0] = $2;
	    }
          | INLINE sigOp fromPartOpt
            {
	      // printf("decl8\n");
	      print_overlay( @$.first_column, @$.last_column, "declaration");
	      $$ = newDeclNode( InlineK );
	      $$->child[0] = $2;
	    }
          | IMPORT sigOp fromPartOpt
            {
	      // printf("decl9\n");
	      print_overlay( @$.first_column, @$.last_column, "declaration");
	      $$ = newDeclNode( ImportK );
	      $$->child[0] = $2;
	    }
          | EXPORT sigOp
            {
	      // printf("decl10\n");
	      print_overlay( @$.first_column, @$.last_column, "declaration");
	      $$ = newDeclNode( ExportK );
	      $$->child[0] = $2;
	    }
          | EXPORT sigOp toPart
            {
	      // printf("decl11\n");
	      print_overlay( @$.first_column, @$.last_column, "declaration");
	      $$ = newDeclNode( ExportK );
	      $$->child[0] = $2;
	      $$->child[1] = $3;
	    }
          | EXPORT sigOp fromPart
            {
	      // printf("decl12\n");
	      print_overlay( @$.first_column, @$.last_column, "declaration");
	      $$ = newDeclNode( ExportK );
	      $$->child[0] = $2;
	      $$->child[1] = $3;
	    }

toPart:
            TO infixed               
            {
	     // printf("toPart\n");
	      print_overlay( @$.first_column, @$.last_column, "toPart");
	      $$ = newStmtNode(ToK);
	      $$->child[0] = $2;
	    }
fromPartOpt:                             
            fromPart
            {
	     // printf("fromPartOpt1\n");
		print_overlay( @$.first_column, @$.last_column, "fromPartOpt");
	      $$ = $1;
	    }        
          | /* empty */
            {
	     // printf("fromPartOpt2\n");
		print_overlay( @$.first_column, @$.last_column, "fromPartOpt");
	      $$ = NULL;   //Tree = NULL;
	    }

fromPart:
            FROM infixedCL
            {
	     // printf("fromPart\n");
		print_overlay( @$.first_column, @$.last_column, "fromPart");
	      $$ = newExpNode( OpK );
	      $$->attr.op = FROM;
	      $$->child[0] = $2;
	    }

infixedCL:
            infixed
            {
	     // printf("infixedCL1\n");
		print_overlay( @$.first_column, @$.last_column, "infixedCL");
	      $$ = $1;
	    }
          | infixedCL COMMA infixed
            {
	     // printf("infixedCL2\n");
		print_overlay( @$.first_column, @$.last_column, "infixedCL");
	      $$ = newTempNode(TK);
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

sigOp:
            sig
            {
	     // printf("sigOp1\n");
		print_overlay( @$.first_column, @$.last_column, "sigOp");
	      $$ = $1;
	    }
          | /* empty */
            {
	     // printf("sigOp-empty\n");
		print_overlay( @$.first_column, @$.last_column, "sigOp");
	      $$ = NULL;   //Tree = NULL;
	    }

sig:
            declBinding
            {
	     // printf("sig-declBinding\n");
		print_overlay( @$.first_column, @$.last_column, "sig");
	      $$ = $1;
	    }
          | block
            {
	     // printf("sig-block\n");
		print_overlay( @$.first_column, @$.last_column, "sig");
	      $$ = $1;
	    }

declPart:
            COLON type
            {
	     // printf("declPart1\n");
		print_overlay( @$.first_column, @$.last_column, "declPart");
	      $$ = newExpNode(OpK);
	      $$->attr.op = COLON;
	      $$->child[0] = $2;
	    }                 
          | COLON_STAR type
            {
	     // printf("declPart2\n");
		print_overlay( @$.first_column, @$.last_column, "declPart");
	      $$ = newExpNode(OpK);
	      $$->attr.op = COLON_STAR;
	      $$->child[0] = $2;
	    }

comma:
	    commaItem
            {
	       // printf("comma1\n");
		print_overlay( @$.first_column, @$.last_column, "comma");
	      $$ = $1;
	    }
          | commaItem COMMA comma
            {
	       // printf("comma2\n");
		print_overlay( @$.first_column, @$.last_column, "comma");
/* 	      if( $$->kind.exp == IdK)          //ID 0 or 1 */
/* 		$$ = newExpNode(ConstK); */
/* 	      else if( $$->kind.exp == ConstK)  //CONSTANT */
/* 		$$ = newExpNode(IdK); */
/* 	      else */
/* 		$$ = newNode(); */

		$$ = newTempNode(TK);
		$$->child[0] = $1;
		$$->child[1] = $3;

/* 	      $$->attr.name = $2->attr.name; */
/* 	      free($2); */
/* 	      $$->sibling = $4; */

/* 	      // printf("FIRST:\n");
/*       	 printTree($1); */
/* 	      // printf("LAST:\n"); */
/* 	      printTree($3); */
	    
/* 	      // printf("TOTAL:\n"); */
/* 	      printTree($$); */

	    }       

commaItem:
            binding_AS
            {
	      // printf("commaItem1\n");
		print_overlay( @$.first_column, @$.last_column, "commaItem");
	      $$ = $1;
	      // printTree($$);
	    }        
          | binding_AS WHERE commaItem
            {
	      // printf("commaItem2\n");
		print_overlay( @$.first_column, @$.last_column, "commaItem");
	      $$ = newStmtNode( WhereK );
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

declBinding:
            bindingR_IED_AS
            {
	    // printf("declBinding\n");
	      print_overlay( @$.first_column, @$.last_column, "declBinding");
	      $$ = $1;
	    }        

infixedExprsDecl:
            infixedExprs
            {
	    // printf("infixedExprsDecl1\n");
	      print_overlay( @$.first_column, @$.last_column, "infixedExprsDecl");
	      $$ = $1;
	    }         
          | infixedExprs declPart       
            {
	    // printf("infixedExprsDecl2\n");
	      print_overlay( @$.first_column, @$.last_column, "infixedExprsDecl");
	      $$ = $1;
	      $$->child[0] = $2;
	    }


infixedExprs:
	    infixedExpr                 
            {
	    // printf("infixedExprs1\n");
	      print_overlay( @$.first_column, @$.last_column, "infixedExprs");
	      $$ = $1;
	    }
          | infixedExpr COMMA infixedExprs
            {
	    // printf("infixedExprs2\n");
	      print_overlay( @$.first_column, @$.last_column, "infixedExprs");
	      $$ = newTempNode(TK);
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

dgc_Binding_Symbol:
            IS_ASSIGNED
            {
	    // printf("dgc_Binding_Symbol1\n");
	      print_overlay( @$.first_column, @$.last_column, "dgc_Binding_Symbol");
	      $$ = newExpNode(OpK);
	      $$->attr.op = IS_ASSIGNED;
	    }        
          | VERY_EQUAL
            {
	    // printf("dgc_Binding_Symbol2\n");
		print_overlay( @$.first_column, @$.last_column, "dgc_Binding_Symbol");
	      $$ = newExpNode(OpK);
	      $$->attr.op = VERY_EQUAL;
	    }  
          | MACRO_IS                    
            {
	    // printf("dgc_Binding_Symbol3\n");
		print_overlay( @$.first_column, @$.last_column, "dgc_Binding_Symbol");
	      $$ = newExpNode(OpK);
	      $$->attr.op = MACRO_IS;
	    }  
          | FUN_ARROW                   
            {
	    // printf("dgc_Binding_Symbol4\n");
		print_overlay( @$.first_column, @$.last_column, "dgc_Binding_Symbol");
	      $$ = newExpNode(OpK);
	      $$->attr.op = FUN_ARROW;
	    }  


bindingR_IED_AS:
	    infixedExprsDecl
            {
	    // printf("bindingR_IED_AS1\n");
		print_overlay( @$.first_column, @$.last_column, "bindingR_IED_AS:");
	      $$ = $1;
	    }
          | infixedExprsDecl dgc_Binding_Symbol binding_AS
            {
	    // printf("bindingR_IED_AS2\n");
		print_overlay( @$.first_column, @$.last_column, "bindingR_IED_AS:");
	      $$ = $2;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

binding_AS:
            bindingL_Infixed_AS 
            {
	    // printf("binding_AS\n");
		print_overlay( @$.first_column, @$.last_column, "binding_AS");
	      $$ = $1;
	      //printTree($1);
	      //printTree($$);
	    }
        

binding_BS:
            bindingL_Infixed_BS         
            {
	    // printf("binding_BS\n");
		print_overlay( @$.first_column, @$.last_column, "binding_BS");
	      $$ = $1;
	    }

binding_Collection:
            bindingL_Infixed_Collection 
            {
	    // printf("binding_Collection\n");
		print_overlay( @$.first_column, @$.last_column, "binding_Collection");
	      $$ = $1;
	    }

bindingL_Infixed_AS:
            anyStatement
            {
	    // printf("bindingL_Infixed_As1\n");
		print_overlay( @$.first_column, @$.last_column, "bindingL_Infixed_AS:");
	      $$ = $1;
	    }                
          | infixed dgc_Binding_Symbol binding_AS
            {
	    // printf("bindingL_Infixed_As2\n");
		print_overlay( @$.first_column, @$.last_column, "bindingL_Infixed_AS:");
	      $$ = $2;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

bindingL_Infixed_Collection:
            collection
            {
	    // printf("bindingL_Infixed_Collection1\n");
		print_overlay( @$.first_column, @$.last_column, "bindingL_Infixed_Collection");
	      $$ = $1;
	    }        
          | infixed dgc_Binding_Symbol binding_Collection
            {
	    // printf("bindingL_Infixed_Collection2\n");
		print_overlay( @$.first_column, @$.last_column, "bindingL_Infixed_Collection");
	      $$ = $2;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

bindingL_Infixed_BS:
            balStatement
            {
	    // printf("bindingL_Infixed_BS1\n");
		print_overlay( @$.first_column, @$.last_column, "bindingL_Infixed_BS");
	      $$ = $1;
	    }        
          | infixed dgc_Binding_Symbol binding_BS
            {
	    // printf("bindingL_Infixed_BS2\n");
		print_overlay( @$.first_column, @$.last_column, "bindingL_Infixed_BS");
	      $$ = $2;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

anyStatement:
            IF commaItem THEN binding_AS
            {
	    // printf("anyStatement1\n");
		print_overlay( @$.first_column, @$.last_column, "anyStatement");
	      $$ = newStmtNode(IfK);
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	    }
          | flow_as                     
            {
	    // printf("anyStatement2\n");
		print_overlay( @$.first_column, @$.last_column, "anyStatement");
	      $$ = $1;
	    }

balStatement:
            flow_bs                  
            {
	    // printf("balStatement\n");
		print_overlay( @$.first_column, @$.last_column, "balStatement");
	      $$ = $1;
	    }

flow_as:  
            collection
            {
	    // printf("flow_as1\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = $1;
	    }
          | IF commaItem
            THEN binding_BS
            ELSE binding_AS
            {
	    // printf("flow_as2\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(IfK);
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	      $$->child[2] = $6;
	    }
          | collection EXIT binding_AS
            {
	    // printf("flow_as3\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode( ExitK);
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | iterators REPEAT binding_AS
            {
	    // printf("flow_as4\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(RepeatK);
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | TRY binding_AS CATCH casesOpt finalPart_AS
            {
	    // printf("flow_as5\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(TryK);
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	      $$->child[2] = $5;
	    }
          | TRY binding_AS CATCH casesOpt alwaysPart_AS
            {
	    // printf("flow_as6\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(TryK);
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	      $$->child[2] = $5;
	    }
          | SELECT binding_AS IN cases    
            {
	    // printf("flow_as7\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(SelectK);
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	    }
          | DO binding_AS                    
            {
	    // printf("flow_as8\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(DoK);
	      $$->child[0] = $2;
	    }
          | DELAY binding_AS                 
            {
	    // printf("flow_as9\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(DelayK);
	      $$->child[0] = $2;
	    }
          | GENERATE genBound binding_AS     
            {
	    // printf("flow_as10\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(GenerateK);
	      $$->child[0] = $2;
	      $$->child[1] = $3;
	    }
          | ASSERT binding_AS                
            {
	    // printf("flow_as11\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(AssertK);
	      $$->child[0] = $2;
	    }
          | ITERATE                          
            {
	    // printf("flow_as12\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(IterateK);
	    }
          | ITERATE name                     
            {
	    // printf("flow_as13\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(IterateK);
	      $$->child[0] = $2;
	    }
          | BREAK                            
            {
	    // printf("flow_as14\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(BreakK);
	    }
          | BREAK name                       
            {
	    // printf("flow_as15\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(BreakK);
	      $$->child[0] = $2;
	    }
          | RETURN
            {
	    // printf("flow_as16\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(ReturnK);
	    }
          | RETURN collection                
            {
	    // printf("flow_as17\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(ReturnK);
	      $$->child[0] = $2;
	    }
          | YIELD binding_AS                 
            {
	    // printf("flow_as18\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(YieldK);
	      $$->child[0] = $2;
	    }
          | EXCEPT binding_AS                
            {
	    // printf("flow_as19\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(ExceptK);
	      $$->child[0] = $2;
	    }
          | GOTO id                          
            {
	    // printf("flow_as20\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(GotoK);
	      $$->child[0] = $2;
	    }
          | NEVER
            {
	    // printf("flow_as21\n");
		print_overlay( @$.first_column, @$.last_column, "flow_as");
	      $$ = newStmtNode(NeverK);
	    }

flow_bs   : collection                  
            {
	    // printf("flow_bs1\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = $1;
	    }
          | IF commaItem
            THEN binding_BS
            ELSE binding_BS
            {
	    // printf("flow_bs2\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(IfK);
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	      $$->child[2] = $6;
	    }
          | collection EXIT binding_BS  
            {
	    // printf("flow_bs3\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(ExitK);
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | iterators REPEAT binding_BS
            {
	    // printf("flow_bs4\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(RepeatK);
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | TRY binding_AS CATCH casesOpt finalPart_BS
            {
	    // printf("flow_bs5\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(TryK);
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	      $$->child[2] = $5;
	    }
          | TRY binding_AS CATCH casesOpt alwaysPart_BS
            {
	    // printf("flow_bs6\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(TryK);
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	      $$->child[2] = $5;
	    }
          | SELECT binding_AS IN cases    
            {
	    // printf("flow_bs7\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(SelectK);
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	    }
          | DO binding_BS                    
            {
	    // printf("flow_bs8\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(DoK);
	      $$->child[0] = $1;
	    }
          | DELAY binding_BS                 
            {
	    // printf("flow_bs9\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(DelayK);
	      $$->child[0] = $2;
	    }
          | GENERATE genBound binding_BS     
            {
	    // printf("flow_bs10\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(GenerateK);
	      $$->child[0] = $2;
	      $$->child[1] = $3;
	    }
          | ASSERT binding_BS                
            {
	    // printf("flow_bs11\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(AssertK);
	      $$->child[0] = $2;
	    }
          | ITERATE
            {
	    // printf("flow_bs12\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(IterateK);
	    }
          | ITERATE name                     
            {
	    // printf("flow_bs13\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(IterateK);
	      $$->child[0] = $2;
	    }
          | BREAK
            {
	    // printf("flow_bs14\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(BreakK);
	    }
          | BREAK name                       
            {
	    // printf("flow_bs15\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(BreakK);
	      $$->child[0] = $2;
	    }
          | RETURN                           
            {
	    // printf("flow_bs16\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(ReturnK);
	    }
          | RETURN collection                
            {
	    // printf("flow_bs17\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(ReturnK);
	      $$->child[0] = $2;
	    }
          | YIELD binding_BS                 
            {
	    // printf("flow_bs18\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(YieldK);
	      $$->child[0] = $2;
	    }
          | EXCEPT binding_BS               
            {
	    // printf("flow_bs19\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(ExceptK);
	      $$->child[0] = $2;
	    }
          | GOTO id                          
            {
	    // printf("flow_bs20\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(GotoK);
	      $$->child[0] = $2;
	    }
          | NEVER
            {
	    // printf("flow_bs21\n");
		print_overlay( @$.first_column, @$.last_column, "flow_bs");
	      $$ = newStmtNode(NeverK);
	    }

genBound: 
            TO commaItem OF
            {
	    // printf("genBound1\n");
		print_overlay( @$.first_column, @$.last_column, "genBound");
	      $$ = newStmtNode(ToK);
	      $$->child[0] = $2;
	    }
	  | /* empty */
            {
	    // printf("genBound-empty\n");
		print_overlay( @$.first_column, @$.last_column, "genBound");
	      $$ = NULL;   //Tree = NULL;
	    }

cases:
            binding_Collection
            {
	    // printf("case\n");
		print_overlay( @$.first_column, @$.last_column, "cases");
	      $$ = $1;
	    }          

casesOpt: 
            binding_Collection
            {
	    // printf("caseOpt1\n");
		print_overlay( @$.first_column, @$.last_column, "casesOpt");
	      $$ = $1;
	    }                  
	  | /* empty */
            {
	    // printf("caseOpt-empty\n");
		print_overlay( @$.first_column, @$.last_column, "casesOpt");
	      $$ = NULL;   //Tree = NULL;
	    }

alwaysPart_AS:
	    ALWAYS binding_AS
            {
	    // printf("always_AS\n");
		print_overlay( @$.first_column, @$.last_column, "alwaysPart_AS");
	      $$ = newStmtNode(AlwaysK);
	      $$->child[0] = $1;
	    }

alwaysPart_BS:
            ALWAYS binding_BS
            {
	    // printf("always_BS\n");
		print_overlay( @$.first_column, @$.last_column, "alwaysPart_BS");
	      $$ = newStmtNode(AlwaysK);
	      $$->child[0] = $1;
	    }

finalPart_BS:
            FINALLY binding_BS
            {
	    // printf("final_BS\n");
		print_overlay( @$.first_column, @$.last_column, "finalPart_BS");
	      $$ = newStmtNode(FinallyK);
	      $$->child[0] = $2;
	    }
          | /* empty */
            {
	    // printf("\n");
		print_overlay( @$.first_column, @$.last_column, "finalPart_BS");
	      $$ = NULL;   //Tree = NULL;
	    }

finalPart_AS:
            FINALLY binding_AS 
            {
	    // printf("final_AS\n");
		print_overlay( @$.first_column, @$.last_column, "finalPart_AS");
	      $$ = newStmtNode(FinallyK);
	      $$->child[0] = $2;
	    }
          | /* empty */
            {
	    // printf("\n");
		print_overlay( @$.first_column, @$.last_column, "finalPart_AS");
	      $$ = NULL;   //Tree = NULL;
	    }


collection:
            infixed
            {
	    // printf("collection-infixed\n");
		print_overlay( @$.first_column, @$.last_column, "collection");
	      $$ = $1;
	    }
          | infixed iterators           
            {
	    // printf("collection-infixed + iter\n");
		print_overlay( @$.first_column, @$.last_column, "collection");
	      $$ = newTempNode(TK);
	      $$->child[0] = $1;
	      $$->child[1] = $2;
	    }

iterators:
            iterators1
            {
	    // printf("iterators\n");
		print_overlay( @$.first_column, @$.last_column, "iterators");
	      $$ = $1;
	    }
        
iterators1:
            iterator  
            {
	    // printf("iterators1-1\n");
		print_overlay( @$.first_column, @$.last_column, "iterators1");
	      $$ = $1;
	    }                  
          | iterators1 iterator         
            {
	    // printf("iterators1-2\n");
		print_overlay( @$.first_column, @$.last_column, "iterators1");
	      $$ = newTempNode(TK);
	      $$->child[0] = $1;
	      $$->child[1] = $2;
	    }

iterator:
	    FOR forLHS IN infixed
            {
	    // printf("itr1\n");
		print_overlay( @$.first_column, @$.last_column, "iterator");
	      $$ = newStmtNode(ForK);
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	    } 
          | FOR forLHS IN infixed suchThatPart
            {
	    // printf("itr2\n");
		print_overlay( @$.first_column, @$.last_column, "iterator");
	      $$ = newStmtNode(ForK);
	      $$->child[0] = $2;
	      $$->child[1] = $4;
	      $$->child[2] = $5;
	    }
          | WHILE infixed
            {
	    // printf("itr3\n");
		print_overlay( @$.first_column, @$.last_column, "iterator");
	      $$ = newStmtNode(WhileK);
	      $$->child[0] = $2;
	    }

forLHS:
            infixed
            {
	    // printf("forLHS1\n");
		print_overlay( @$.first_column, @$.last_column, "forLHS");
	      $$ = $1;
	    }
          | FREE infixed
            {
	    // printf("forLHS2\n");
		print_overlay( @$.first_column, @$.last_column, "forLHS");
	      $$ = newDeclNode( FreeK );
	      $$->child[0] = $2;
	    }             
          | LOCAL infixed
            {
	    // printf("forLHS3\n");
		print_overlay( @$.first_column, @$.last_column, "forLHS");
	      $$ = newDeclNode( LocalK );
	      $$->child[0] = $2;
	    }
          | FLUID infixed
            {
	    // printf("forLHS4\n");
		print_overlay( @$.first_column, @$.last_column, "forLHS");
	      $$ = newDeclNode( FluidK );
	      $$->child[0] = $2;
	    }

suchThatPart:
            BAR infixed
            {
	    // printf("suchThatPart\n");
		print_overlay( @$.first_column, @$.last_column, "suchThatPart");
	      $$ = newExpNode(OpK);
	      $$->attr.op = BAR;
	      $$->child[0] = $2; 
	    }

infixed:
	    infixedExpr
            {
	    // printf("infixed1\n");
		print_overlay( @$.first_column, @$.last_column, "infixed");
	      $$ = $1;
	    }
          | infixedExpr declPart
            {
	    // printf("infixed2\n");
		print_overlay( @$.first_column, @$.last_column, "infixed");
	      $$ = newTempNode(TK);
	      $$->child[0] = $1;
	      $$->child[1] = $2;
	    }        
          | block
            {
	    // printf("infixed3\n");
		print_overlay( @$.first_column, @$.last_column, "infixed");
	      $$ = $1;
	    }        

infixedExpr:
            e11_OP
            {
	    // printf("infixedExpr1\n");
		print_overlay( @$.first_column, @$.last_column, "infixedExpr");
	      $$ = $1;
	    }
          | e3
            {
	    // printf("infixedExpr2\n");
		print_overlay( @$.first_column, @$.last_column, "infixedExpr");
	      $$ = $1;
	    }

e3:
            e4
            {
	    // printf("e3-1\n");
		print_overlay( @$.first_column, @$.last_column, "e3");
	      $$ = $1;
	    }                  
          | e3 AND e4 
            {
	    // printf("e3-2\n");
		print_overlay( @$.first_column, @$.last_column, "e3");
	      $$ = newStmtNode(AndK);
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }               
          | e3 OR e4
            {
	    // printf("e3-3\n");
		print_overlay( @$.first_column, @$.last_column, "e3");
	      $$ = newStmtNode(OrK);
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }    
          | e3 latticeOp e4
            {
	    // printf("e3-4\n");
		print_overlay( @$.first_column, @$.last_column, "e3");
	      $$ = $2;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }


e4:
            e5                          
            {
	    // printf("e4-1\n");
		print_overlay( @$.first_column, @$.last_column, "e4");
	      $$ = $1;
	    }
          | e4 HAS e5                
            {
	    // printf("e4-2\n");
		print_overlay( @$.first_column, @$.last_column, "e4");
	      $$ = newStmtNode(HasK);
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | e4 relationOp e5            
            {
	    // printf("e4-3\n");
		print_overlay( @$.first_column, @$.last_column, "e4");
	      $$ = $2;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | relationOp e5               
            {
	    // printf("e4-4\n");
		print_overlay( @$.first_column, @$.last_column, "e4");
	      $$ = $1;
	      $$->child[0] = $2;
	    }

e5:
            e6                          
            {
	    // printf("e5-1\n");
		print_overlay( @$.first_column, @$.last_column, "e5");
	      $$ = $1;
	    }
          | e5 seqOp                    
            {
	    // printf("e5-2\n");
		print_overlay( @$.first_column, @$.last_column, "e5");
	      $$ = $2;
	      $$->child[0] = $1;
	    }
          | e5 seqOp e6
            {
	    // printf("e5-3\n");
		print_overlay( @$.first_column, @$.last_column, "e5");
	      $$ = $2;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

e6:
            e7                          
            {
	    // printf("e6-1\n");
		print_overlay( @$.first_column, @$.last_column, "e6");
	      $$ = $1;
	    }
          | e6 plusOp e7                
            {
	    // printf("e6-2\n");
		print_overlay( @$.first_column, @$.last_column, "e6");
	      $$ = $2;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | plusOp e7                   
            {
	    // printf("e6-3\n");
		print_overlay( @$.first_column, @$.last_column, "e6");
	      $$ = $1;
	      $$->child[0] = $2;
	    }
e7:
            e8
            {
	    // printf("e7-1\n");
		print_overlay( @$.first_column, @$.last_column, "e7");
	      $$ = $1;
	    }
          | e7 quotientOp e8            
            {
	    // printf("e7-2\n");
		print_overlay( @$.first_column, @$.last_column, "e7");
	      $$ = $2;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

e8:
            e9
	    {
	    // printf("e8-1\n");
		print_overlay( @$.first_column, @$.last_column, "e8");
	      $$ = $1;
	    }
          | e8 timesOp e9               
            {
	    // printf("e8-2\n");
		print_overlay( @$.first_column, @$.last_column, "e8");
	      $$ = $2;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

e9:         
            e11_e12
            {
	    // printf("e9-1\n");
		print_overlay( @$.first_column, @$.last_column, "e9");
	      $$ = $1;
	    }
          | e11_e12 powerOp e9          
            {
	    // printf("e9-2\n");
		print_overlay( @$.first_column, @$.last_column, "e9");
	      $$ = $2;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

e11_OP:
            op                          
            {
	    // printf("e11_OP1\n");
		print_overlay( @$.first_column, @$.last_column, "e11_OP");
	      $$ = $1;
	    }
          | e11_OP DCOLON e12           
	    {
	    // printf("e11_OP2\n");
		print_overlay( @$.first_column, @$.last_column, "e11_OP");
	      $$ = newExpNode(OpK);
	      $$->attr.op = DCOLON;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | e11_OP ATSIGN e12           
            {
	    // printf("e11_OP3\n");
		print_overlay( @$.first_column, @$.last_column, "e11_OP");
	      $$ = newExpNode(OpK);
	      $$->attr.op = ATSIGN;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | e11_OP PRETEND e12       
            {
	    // printf("e11-OP4\n");
		print_overlay( @$.first_column, @$.last_column, "e11_OP");
	      $$ = newStmtNode(PretendK);
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

e11_e12:
            e12                         
            {
	    // printf("e11_e12-1\n");
		print_overlay( @$.first_column, @$.last_column, "e11_e12");
	      $$ = $1;
	    }
          | e11_e12 DCOLON e12          
            {
	    // printf("e11_e12-2\n");
		print_overlay( @$.first_column, @$.last_column, "e11_e12");
	      $$ = newExpNode(OpK);
	      $$->attr.op = DCOLON;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | e11_e12 ATSIGN e12          
            {
	    // printf("e11_e12-3\n");
		print_overlay( @$.first_column, @$.last_column, "e11_e12");
	      $$ = newExpNode(OpK);
	      $$->attr.op = ATSIGN;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | e11_e12 PRETEND e12      
            {
	    // printf("e11_e12-4\n");
		print_overlay( @$.first_column, @$.last_column, "e11_e12");
	      $$ = newStmtNode(PretendK);
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }


type:
            e11_e12
            {
	    // printf("type\n");
		print_overlay( @$.first_column, @$.last_column, "type");
	      $$ = $1;
	    }

e12:
            e13
            {
	    // printf("e12-1\n");
		print_overlay( @$.first_column, @$.last_column, "e12");
	      $$ = $1;
	    }
          | e13 arrowOp e12             
            {
	    // printf("e12-2\n");
		print_overlay( @$.first_column, @$.last_column, "e12");
	      $$ = $2;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

e13:
            e14
            {
	    // printf("e13-1\n");
		print_overlay( @$.first_column, @$.last_column, "e13");
	      $$ = $1;
	    }
          | e14 DOLLAR qualTail
            {
	    // printf("e13-2\n");
		print_overlay( @$.first_column, @$.last_column, "e13");
	      $$ = newExpNode(OpK);
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

qualTail: 
            leftJuxtaposed
            {
	    // printf("qualTail1\n");
		print_overlay( @$.first_column, @$.last_column, "qualTail");
	      $$ = $1;
	    }
          | leftJuxtaposed DOLLAR qualTail
            {
	    // printf("qualTail2\n");
		print_overlay( @$.first_column, @$.last_column, "qualTail");
	      $$ = newExpNode(OpK);
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
                      
opQualTail:
	    molecule                    
            {
	    // printf("opQualTail1\n");
		print_overlay( @$.first_column, @$.last_column, "opQualTail");
	      $$ = $1;
	    }
          | molecule DOLLAR opQualTail
            {
	    // printf("opQualTail2\n");
		print_overlay( @$.first_column, @$.last_column, "opQualTail");
	      $$ = newExpNode(OpK);
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

e14:
            e15
            {
	    // printf("e14-1\n");
		print_overlay( @$.first_column, @$.last_column, "e14");
	      $$ = $1;
	    }
          | e14 EXCEPT e15           
            {
	    // printf("e14-2\n");
		print_overlay( @$.first_column, @$.last_column, "e14");
	      $$ = newStmtNode(ExceptK);
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | WITH declMolecule        
            {
	    // printf("e14-3\n");
		print_overlay( @$.first_column, @$.last_column, "e14");
	      $$ = newStmtNode(WithK);
	      $$->child[0] = $2;
	    }
          | e14 WITH declMolecule    
            {
	    // printf("e14-4\n");
		print_overlay( @$.first_column, @$.last_column, "e14");
	      $$ = newStmtNode(WithK);
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
          | ADD declMolecule         
            {
	    // printf("e14-5\n");
		print_overlay( @$.first_column, @$.last_column, "e14");
	      $$ = newStmtNode(AddK);
	      $$->child[0] = $2;
	    }
          | e14 ADD declMolecule     
            {
	    // printf("e14-6\n");
		print_overlay( @$.first_column, @$.last_column, "e14");
	      $$ = newStmtNode(AddK);
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

e15:
            application                 
            {
	    // printf("e15\n");
		print_overlay( @$.first_column, @$.last_column, "e15");
	      $$ = $1;
	    }
op:
            arrowOp
            {
	    // printf("op1\n");
		print_overlay( @$.first_column, @$.last_column, "op");
	      $$ = $1;
	    }
          | latticeOp                  
            {
	    // printf("op2\n");
		print_overlay( @$.first_column, @$.last_column, "op");
	      $$ = $1;
	    }
          | relationOp                  
            {
	    // printf("op3\n");
		print_overlay( @$.first_column, @$.last_column, "op");
	      $$ = $1;
	    }
          | seqOp                       
            {
	    // printf("op4\n");
		print_overlay( @$.first_column, @$.last_column, "op");
	      $$ = $1;
	    }
          | plusOp                      
            {
	    // printf("op5\n");
		print_overlay( @$.first_column, @$.last_column, "op");
	      $$ = $1;
	    }
          | quotientOp                  
            {
	    // printf("op6\n");
		print_overlay( @$.first_column, @$.last_column, "op");
	      $$ = $1;
	    }
          | timesOp                     
            {
	    // printf("op7\n");
		print_overlay( @$.first_column, @$.last_column, "op");
	      $$ = $1;
	    }
          | powerOp                     
            {
	    // printf("op8\n");
		print_overlay( @$.first_column, @$.last_column, "op");
	      $$ = $1;
	    }

nakedOp:
            arrowTok                    
            {
	    // printf("nakedOp1\n");
		print_overlay( @$.first_column, @$.last_column, "nakedOp");
	      $$ = $1;
	    }
          | latticeTok                  
            {
	    // printf("nakedOp2\n");
		print_overlay( @$.first_column, @$.last_column, "nakedOp");
	      $$ = $1;
	    }
          | relationTok                 
            {
	    // printf("nakedOp3\n");
		print_overlay( @$.first_column, @$.last_column, "nakedOp");
	      $$ = $1;
	    }
          | seqTok                      
            {
	    // printf("nakedOp4\n");
		print_overlay( @$.first_column, @$.last_column, "nakedOp");
	      $$ = $1;
	    }
          | plusTok                     
            {
	    // printf("nakedOp5\n");
		print_overlay( @$.first_column, @$.last_column, "nakedOp");
	      $$ = $1;
	    }
          | quotientTok                 
            {
	    // printf("nakedOp6\n");
		print_overlay( @$.first_column, @$.last_column, "nakedOp");
	      $$ = $1;
	    }
          | timesTok                    
            {
	    // printf("nakedOp7\n");
		print_overlay( @$.first_column, @$.last_column, "nakedOp");
	      $$ = $1;
	    }
          | powerTok                    
            {
	    // printf("nakedOp8\n");
		print_overlay( @$.first_column, @$.last_column, "nakedOp");
	      $$ = $1;
	    }

arrowOp:
            arrowTok                    
            {
	    // printf("arrowOp1\n");
		print_overlay( @$.first_column, @$.last_column, "arrowOp");
	      $$ = $1;
	    }
          | arrowTok DOLLAR opQualTail  
            {
	    // printf("arrowOp2\n");
		print_overlay( @$.first_column, @$.last_column, "arrowOp");
	      $$ = newExpNode(OpK);
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
latticeOp:
            latticeTok                  
            {
	    // printf("latticeOp1\n");
		print_overlay( @$.first_column, @$.last_column, "latticeOp");
	      $$ = $1;
	    }
          | latticeTok DOLLAR opQualTail 
            {
	    // printf("latticeOp2\n");
		print_overlay( @$.first_column, @$.last_column, "latticeOp");
	      $$ = newExpNode(OpK);
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

relationOp:
            relationTok                 
            {
	    // printf("relationOp1\n");
		print_overlay( @$.first_column, @$.last_column, "relationOp");
	      $$ = $1;
	    }
          | relationTok DOLLAR opQualTail 
            {
	    // printf("relationOp2\n");
		print_overlay( @$.first_column, @$.last_column, "relationOp");
	      $$ = newExpNode(OpK);
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

seqOp:
            seqTok                      
            {
	    // printf("seqOp1\n");
		print_overlay( @$.first_column, @$.last_column, "seqOp");
	      $$ = $1;
	    }
          | seqTok DOLLAR opQualTail    
            {
	    // printf("seqOp2\n");
		print_overlay( @$.first_column, @$.last_column, "seqOp");
	      $$ = newExpNode(OpK);
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

plusOp:
            plusTok                     
            {
	    // printf("plusOp1\n");
		print_overlay( @$.first_column, @$.last_column, "plusOp");
	      $$ = $1;
	    }
          | plusTok DOLLAR opQualTail   
            {
	    // printf("plusOp2\n");
		print_overlay( @$.first_column, @$.last_column, "plusOp");
	      $$ = newExpNode(OpK);
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

quotientOp:
            quotientTok               
            {
	    // printf("quotientOp1\n");
		print_overlay( @$.first_column, @$.last_column, "quotientOp");
	      $$ = $1;
	    }
          | quotientTok DOLLAR opQualTail 
            {
	    // printf("quotientOp2\n");
		print_overlay( @$.first_column, @$.last_column, "quotientOp");
	      $$ = newExpNode(OpK);
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

timesOp:
            timesTok                    
            {
	    // printf("timesOp1\n");
		print_overlay( @$.first_column, @$.last_column, "timesOp");
	      $$ = $1;
	    }
          | timesTok DOLLAR opQualTail  
            {
	    // printf("timesOp2\n");
		print_overlay( @$.first_column, @$.last_column, "timesOp");
	      $$ = newExpNode(OpK);
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
powerOp:
            powerTok                    
            {
	    // printf("powerOp1\n");
		print_overlay( @$.first_column, @$.last_column, "powerOp");
	      $$ = $1;
	    }
          | powerTok DOLLAR opQualTail  
            {
	    // printf("powerOp2\n");
		print_overlay( @$.first_column, @$.last_column, "powerOp");
	      $$ = newExpNode(OpK);
	      $$->attr.op = DOLLAR;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

arrowTok:
            GETS                        
            {
	    // printf("arrowTok-GET\n");
		print_overlay( @$.first_column, @$.last_column, "arrowTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = GETS;
	    }
          | TO_SYMBOL                   
            {
	    // printf("arrowTok-TO_SYMBOL\n");
		print_overlay( @$.first_column, @$.last_column, "arrowTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = TO_SYMBOL;
	    }

latticeTok:
            JOIN                        
            {
	    // printf("latticeTok-JOIN\n");
		print_overlay( @$.first_column, @$.last_column, "latticeTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = JOIN;
	    }
          | MEET                        
            {
	    // printf("latticeTok_MEET\n");
		print_overlay( @$.first_column, @$.last_column, "latticeTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = MEET;
	    }

relationTok:
            ISNT
            {
	    // printf("relationTok-ISNT\n");
		print_overlay( @$.first_column, @$.last_column, "relationTok");
	      $$ = newStmtNode(IsntK);
	    }
          | IS
            {
	    // printf("relationTok-IS\n");
		print_overlay( @$.first_column, @$.last_column, "relationTok");
	      $$ = newStmtNode(IsK);
	    }
          | CASE
            {
	    // printf("relationTok-CASE\n");
		print_overlay( @$.first_column, @$.last_column, "relationTok");
	      $$ = newStmtNode(CaseK);
	    }
          | VERY_LESS                   
            {
	    // printf("relationTok-VERY_LESS\n");
		print_overlay( @$.first_column, @$.last_column, "relationTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = VERY_LESS;
	    }
          | VERY_GREATER                
            {
	    // printf("relationTok-VERY_GREATER\n");
	      print_overlay( @$.first_column, @$.last_column, "relationTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = VERY_GREATER;
	    }
          | LESSTHAN                        
            {
	    // printf("relationTok-LESSTHAN\n");
		print_overlay( @$.first_column, @$.last_column, "relationTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = LESSTHAN;
	    }
          | LESSEQ                      
            {
	    // printf("relationTok-LESSEQ\n");
		print_overlay( @$.first_column, @$.last_column, "relationTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = LESSEQ;
	    }
          | EQUALS                      
            {
	    // printf("relationTok-EQUALS\n");
		print_overlay( @$.first_column, @$.last_column, "relationTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = EQUALS;
	    }
          | GREATEREQ                   
            {
	    // printf("relationTok-GREATEREQ\n");
		print_overlay( @$.first_column, @$.last_column, "relationTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = GREATEREQ;
	    }
          | GREATERTHAN
            {
	    // printf("relationTok-GREATERTHAN\n");
		print_overlay( @$.first_column, @$.last_column, "relationTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = GREATERTHAN;
	    }
	  | UNEQUALS
            {
	    // printf("relationTok-UNEQUALS\n");
		print_overlay( @$.first_column, @$.last_column, "relationTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = UNEQUALS;
	    }
          | CARETEQUALS
            {
	    // printf("relationTok-CARETEQUALS\n");
		print_overlay( @$.first_column, @$.last_column, "relationTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = CARETEQUALS;
	    }

seqTok:
            BY
            {
	    // printf("seqTok-BY\n");
		print_overlay( @$.first_column, @$.last_column, "seqTok");
	      $$ = newStmtNode(ByK);
	    }
          | DOTDOT                      
            {
	    // printf("seqTok-DOTDOT\n");
		print_overlay( @$.first_column, @$.last_column, "seqTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = DOTDOT;
	    }

plusTok:
            PLUS                        
            {
	    // printf("plusTok-PLUS\n");
		print_overlay( @$.first_column, @$.last_column, "plusTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = PLUS;
	    }
          | MINUS                       
            {
	    // printf("plusTok-MINUS\n");
		print_overlay( @$.first_column, @$.last_column, "plusTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = MINUS;
	    }
          | PLUSMINUS                   
            {
	    // printf("plusTok-PLUSMINUS\n");
		print_overlay( @$.first_column, @$.last_column, "plusTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = PLUSMINUS;
	    }

quotientTok:
            MOD                         
            {
	    // printf("quotientTok-MOD\n");
		print_overlay( @$.first_column, @$.last_column, "quotientTok");
	      $$ = newStmtNode(ModK);
	    }
          | QUO                         
            {
	    // printf("quotientTok-QUO\n");
		print_overlay( @$.first_column, @$.last_column, "quotientTok");
	      $$ = newStmtNode(QuoK);
	    }
          | REM                         
            {
	    // printf("quotientTok-REM\n");
		print_overlay( @$.first_column, @$.last_column, "quotientTok");
	      $$ = newStmtNode(RemK);
	    }

timesTok:
            TIMES                       
            {
	    // printf("timesTok-TIMES\n");
		print_overlay( @$.first_column, @$.last_column, "timesTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = TIMES;
	    }
          | DIVIDE                      
            {
	    // printf("timesTok-DIVIDE\n");
		print_overlay( @$.first_column, @$.last_column, "timesTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = DIVIDE;
	    }
          | SETMINUS                    
            {
	    // printf("timesTok-SETMINUS\n");
		print_overlay( @$.first_column, @$.last_column, "timesTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = SETMINUS;
	    }

powerTok:
            CARET                       
            {
	    // printf("powerTok-CARET\n");
		print_overlay( @$.first_column, @$.last_column, "powerTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = CARET;
	    }
          | STARSTAR                    
            {
	    // printf("powerTok-STARSTAR\n");
		print_overlay( @$.first_column, @$.last_column, "powerTok");
	      $$ = newExpNode(OpK);
	      $$->attr.op = STARSTAR;
	    }

application:
            rightJuxtaposed
            {
	    // printf("application\n");
		print_overlay( @$.first_column, @$.last_column, "application");

	      $$ = $1;
	    }

rightJuxtaposed:
            jRight_Mol            
            {
	    // printf("rightJuxtaposed\n");
		print_overlay( @$.first_column, @$.last_column, "rightJuxtaposed");
	      $$ = $1;
	    }

leftJuxtaposed:
            jLeft_Mol             
            {
	    // printf("leftJuxtaposed\n");
		print_overlay( @$.first_column, @$.last_column, "leftJuxtaposed");
	      $$ = $1;
	    }

jRight_Mol:
            jLeft_Mol                   
            {
	    // printf("jRight_Mol1\n");
		print_overlay( @$.first_column, @$.last_column, "jRight_Mol");
	      $$ = $1;
	    }
          | jLeft_Mol jRight_Atom       
            {
	    // printf("jRight_Mol2\n");
		print_overlay( @$.first_column, @$.last_column, "jRight_Mol");
	      $$ = newTempNode(TK);
	      $$->child[0] = $1;
	      $$->child[1] = $2;
	    }
          | NOT jRight_Atom
            {
	    // printf("jRIght_Mol3\n");
		print_overlay( @$.first_column, @$.last_column, "jRight_Mol");
	      $$ = newStmtNode(NotK);
	      $$->child[0] = $2;
	    }


jRight_Atom:
            jLeft_Atom                 
            {
	    // printf("jRight_Atom1\n");
		print_overlay( @$.first_column, @$.last_column, "jRight_Atom");
	      $$ = $1;
	    }
          | jLeft_Atom jRight_Atom      
            {
	    // printf("jRight_Atom2\n");
		print_overlay( @$.first_column, @$.last_column, "jRight_Atom");
	      $$ = newTempNode(TK);
	      $$->child[0] = $1;
	      $$->child[1] = $2;
	    }
          | NOT jRight_Atom          
            {
	    // printf("jRight_Atom3\n");
		print_overlay( @$.first_column, @$.last_column, "jRight_Atom");
	      $$ = newStmtNode(NotK);
	      $$->child[0] = $2;
	    }

jLeft_Mol :
            molecule                    
            {
	    // printf("jLeft_Mol1\n");
		print_overlay( @$.first_column, @$.last_column, "jLeft_Mol");
	      $$ = $1;
	    }
          | NOT blockEnclosure       
            {
	    // printf("jLeft_Mol2\n")
;		print_overlay( @$.first_column, @$.last_column, "jLeft_Mol");
	      $$ = newStmtNode(NotK);
	      $$->child[0] = $2;
	    }
          | jLeft_Mol blockEnclosure    
            {
	    // printf("jLeft_Mol3\n");
		print_overlay( @$.first_column, @$.last_column, "jLeft_Mol");

	      $$ = newTempNode(TK);
	      $$->child[0] = $1;
	      $$->child[1] = $2;
	    }
          | jLeft_Mol DOT blockMolecule     
            {
	    // printf("jLeft_Mol4\n");
		print_overlay( @$.first_column, @$.last_column, "jLeft_Mol");
	      $$ = newExpNode(OpK);
	      $$->attr.op = DOT;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

jLeft_Atom:
            atom
            {
	    // printf("jLeft_Atom-atom\n");
		print_overlay( @$.first_column, @$.last_column, "jLeft_Atom");
	      $$ = $1;
	    }
          | NOT blockEnclosure       
            {
	    // printf("jLeft_Atom-NOT blockEnc\n");
		print_overlay( @$.first_column, @$.last_column, "jLeft_Atom");
	      $$ = newStmtNode(NotK);
	      $$->child[0] = $2;
	    }
          | jLeft_Atom blockEnclosure   
            {
	    // printf("jLeft_Atom-3\n");
		print_overlay( @$.first_column, @$.last_column, "jLeft_Atom");
	      $$ = newTempNode(TK);
	      $$->child[0] = $1;
	      $$->child[1] = $2;
	    }
          | jLeft_Atom DOT blockMolecule    
            {
	    // printf("jLeft_Atom-4\n");
		print_overlay( @$.first_column, @$.last_column, "jLeft_Atom");
	      $$ = newExpNode(OpK);
	      $$->attr.op = DOT;
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

molecule  :
            atom                        
            {
	    // printf("mol-atom\n");
		print_overlay( @$.first_column, @$.last_column, "molecule");
	      $$ = $1;
	    }
          | enclosure                   
            {
	    // printf("mol-enclosure\n");
		print_overlay( @$.first_column, @$.last_column, "molecule");
	      $$ = $1;
	    }

enclosure :
            parened                     
            {
	    // printf("encl-parened\n");
		print_overlay( @$.first_column, @$.last_column, "enclosure");
	      $$ = $1;
	    }
          | bracketed                   
            {
	    // printf("encl-bracketed\n");
		print_overlay( @$.first_column, @$.last_column, "enclosure");
	      $$ = $1;
	    }
          | quotedIds                   
            {
	    // printf("encl-quotedIds\n");
		print_overlay( @$.first_column, @$.last_column, "enclosure");
	      $$ = $1;
	    }

declMolecule:
            application                 
            {
	    // printf("declMol-appl\n");
		print_overlay( @$.first_column, @$.last_column, "declMolecule");
	      $$ = $1;
	    }
          | block                       
            {
	    // printf("declMol-block\n");
		print_overlay( @$.first_column, @$.last_column, "declMolecule");
	      $$ = $1;
	    }
          | /* empty */
            {
	    // printf("declMol-empty\n");
		print_overlay( @$.first_column, @$.last_column, "declMolecule");
	      $$ = NULL;   //Tree = NULL;
	    }

blockMolecule:
            atom                        
            {
	    // printf("blockMolecule-atom\n");
		print_overlay( @$.first_column, @$.last_column, "blockMolecule");
	      $$ = $1;
	    }
          | enclosure                   
            {
	    // printf("blockMolecule-enclosure\n");
		print_overlay( @$.first_column, @$.last_column, "blockMolecule");
	      $$ = $1;
	    }
          | block                       
            {
	    // printf("blockMolecule-block\n");
		print_overlay( @$.first_column, @$.last_column, "blockMolecule");
	      $$ = $1;
	    }

blockEnclosure:
            enclosure                   
            {
	    // printf("blockEnclosure-enclosure\n");
		print_overlay( @$.first_column, @$.last_column, "blockEnclosure");
	      $$ = $1;
	    }
          | block                       
            {
	    // printf("blockEnclosure-block\n");
		print_overlay( @$.first_column, @$.last_column, "blockEnclosure");
	      $$ = $1;
	    }

block:
            piled_Expression            
            {
	    // printf("block-piled\n");
		print_overlay( @$.first_column, @$.last_column, "block");
	      $$ = $1;
	    }
          | curly_Labelled            
            {
	    // printf("block-curly\n");
		print_overlay( @$.first_column, @$.last_column, "block");
	      $$ = $1;
	    }

parened:
            LPAREN RPAREN               
            {
	    // printf("parened-empty\n");
		print_overlay( @$.first_column, @$.last_column, "parened");
	      $$ = newEncNode(ParenedK);
	    }
          | LPAREN expression RPAREN    
            {
	    // printf("parened-exp\n");
		print_overlay( @$.first_column, @$.last_column, "parened");
	      $$ = newEncNode(ParenedK);
	      $$->child[0] = $2;
	    }

bracketed:
            LBRACK RBRACK               
            {
	    // printf("bracketed-empty\n");
		print_overlay( @$.first_column, @$.last_column, "bracketed");
	      $$ = newEncNode(BracketedK);
	    }
          | LBRACK expression RBRACK
            {
	    // printf("bracketed-exp\n");
		print_overlay( @$.first_column, @$.last_column, "bracketed");
	      $$ = newEncNode(BracketedK);
	      $$->child[0] = $2;
	    }

quotedIds:
            SQUOTE SQUOTE               
            {
	    // printf("quotedIds-empty\n");
		print_overlay( @$.first_column, @$.last_column, "quotedIds");
	      $$ = newEncNode(QuotedIdsK);
	    }
          | SQUOTE names SQUOTE
            {
	    // printf("quotedIds-names\n");
		print_overlay( @$.first_column, @$.last_column, "quotedIds");
	      $$ = newEncNode(QuotedIdsK);
	      $$->child[0] = $2;
	    }

names:
            name                        
            {
	    // printf("names1\n");
		print_overlay( @$.first_column, @$.last_column, "names");
	      $$ = $1;
	    }
          | name COMMA names
            {
	    // printf("names2\n");
		print_overlay( @$.first_column, @$.last_column, "names");
	      $$ = newTempNode(TK);
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }         

atom:  
            id                          
            {
	    // printf("atom-id\n");
		print_overlay( @$.first_column, @$.last_column, "atom");
	      $$ = $1;
	    }
          | literal                     
            {
	    // printf("atom-literal\n");
		print_overlay( @$.first_column, @$.last_column, "atom");
	      $$ = $1;
	    }

id:
            ID 
            {
	    // printf("id-ID  %s\n", tokenString); 
		print_overlay( @$.first_column, @$.last_column, "id");

	      $$ = newExpNode(IdK);
	      $$->attr.name = copyString(tokenString);
	    }
          | POUND
            {
	    // printf("id-POUND\n"); 
		print_overlay( @$.first_column, @$.last_column, "id");

	      $$ = newExpNode(OpK);
	      $$->attr.op = POUND;
	    }
          | TILDE
            {
	    // printf("id-TILDE\n");
		print_overlay( @$.first_column, @$.last_column, "id");
	      $$ = newExpNode(OpK);
	      $$->attr.op = TILDE;
	    }

name:
            ID
            {
	    // printf("name-ID\n");
		print_overlay( @$.first_column, @$.last_column, "name");
	    // printf("POSITION: %d\t%d\n", line_num, init_p);
	      $$ = newExpNode(IdK);
	      $$->attr.name = copyString(tokenString);
	    }
          | nakedOp
            {
	    // printf("name-nakedOp\n");
		print_overlay( @$.first_column, @$.last_column, "name");
	      $$ = $1;
	    }

literal:  
              INT              
              {
	    // printf("literal-INT %s\n", tokenString);
		print_overlay( @$.first_column, @$.last_column, "literal");

		$$ = newExpNode(ConstK);
		$$->attr.name = copyString(tokenString);
	      }
          |   FLOAT                
              { 
	    // printf("literal-FLOAT %s\n", tokenString);
		print_overlay( @$.first_column, @$.last_column, "literal");

		$$ = newExpNode(ConstK);
		$$->attr.name = copyString(tokenString);
	      }
          |   STRING               
              {
	    // printf("literal-STRING  %s\n", tokenString); 
		print_overlay( @$.first_column, @$.last_column, "literal");

		$$ = newExpNode(ConstK);
		$$->attr.name = copyString(tokenString);
	      }

doc_Expression:
            preDocument expression postDocument
            {
	    // printf("dec_Exp\n");
		print_overlay( @$.first_column, @$.last_column, "doc_Expression");
	      $$ = newBlockNode(DocBlockK);
	      $$->child[0] = $1;
	      $$->child[1] = $2;
	      $$->child[2] = $3;
	    }                                        

preDocument:
            preDocumentList             
            {
	    // printf("preDoc\n");
		print_overlay( @$.first_column, @$.last_column, "preDocument");
	      $$ = $1;
	    }

preDocumentList:
            PREDOC {$$ = newName(copyString(tokenString));} 
            preDocumentList
            {
	    // printf("PreDocList1\n");
		print_overlay( @$.first_column, @$.last_column, "preDocumentList");
	      $$ = newExpNode(DocK);
	      $$->attr.name = $2->attr.name;
	      $$->sibling = $3;
	      free($2)
	    }
	  | /* empty */
            {
	      // printf("PreDoc-empty\n");
		print_overlay( @$.first_column, @$.last_column, "preDocumentList");
	      $$ = NULL;   //Tree = NULL;
       	    }

postDocument:
            postDocumentList            
            {
	      // printf("PostDoc\n");
		print_overlay( @$.first_column, @$.last_column, "postDocument");
	      $$ = $1;
	    }

postDocumentList:
            POSTDOC {$$ = newName(copyString(tokenString));} 
            postDocumentList
            {
	      // printf("PostDocList1\n");
		print_overlay( @$.first_column, @$.last_column, "postDocumentList");
	      $$ = newExpNode(DocK);
	      $$->attr.name = $2->attr.name;
	      $$->sibling = $3;
	      free($2);
	    }
          | /* empty */
            {
	      // printf("PostDocList-empty\n");
		print_overlay( @$.first_column, @$.last_column, "postDocumentList");
	      $$ = NULL;   //Tree = NULL;
	    }

piled_Expression:
            SETTAB pileContents_Expression BACKTAB
            {
	      // printf("pile_Exp\n");
		print_overlay( @$.first_column, @$.last_column, "piled_Expression");
	      $$ = newBlockNode(PiledK);
	      $$->child[0] = $2;
	    }

pileContents_Expression:
            doc_Expression              
            {
	      // printf("pileContents_Exp\n");
		print_overlay( @$.first_column, @$.last_column, "pileContents_Expression");
	      $$ = $1;
	    }
          | pileContents_Expression BACKSET doc_Expression
            {
	      // printf("pileContents_Exp2\n");
		print_overlay( @$.first_column, @$.last_column, "pileContents_Expression");
	      $$ = newTempNode(TK);
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }

curly_Labelled:
            LBRACE
            curlyContents_Labelled
            RBRACE
            {
	      // printf("curly_Labelled\n");
		print_overlay( @$.first_column, @$.last_column, "curly_Labelled");
	      $$ = $2;
	    }
                                             
curlyContents_Labelled:
            curlyContentsList_Labelled
            {
	      // printf("curlyContents_Labelled\n");
		print_overlay( @$.first_column, @$.last_column, "curlyContents_Labelled");
	      $$ = newBlockNode(CurlyK);
	      $$->child[0] = $1;
	    }  

curlyContentsList_Labelled:
            curlyContent1_Labelled
            {
	      // printf("curlList_Labelled1\n");
		print_overlay( @$.first_column, @$.last_column, "curlyContentsList_Labelled");
	      $$ = $1;
	    }      
          | curlyContent1_Labelled curlyContentB_Labelled %prec SEMI
            {
	      // printf("curlList_Labelled2\n");
		print_overlay( @$.first_column, @$.last_column, "curlyContentsList_Labelled");

	      if( $1 == NULL )
		$$ = $2;

	      else
	      {
		$$ = newTempNode(TK);
	
		$$->child[0] = $1;
		$$->child[1] = $2;
	      }
	    }

curlyContent1_Labelled:
            curlyContent1_Labelled curlyContentA_Labelled
            {
	      // printf("curl_labelled-1\n");
		print_overlay( @$.first_column, @$.last_column, "curlyContent1_Labelled");
	      if( $1 == NULL )
		$$ = $2;
	  
	      else if( $2 == NULL )
		$$ = $1;

	      else
	      {
		$$ = newTempNode(TK);
		$$->child[0] = $1;
		$$->child[1] = $2;
	      }
	    }
	  | /* empty */
            {
	      // printf("curl1_Labelled-empty\n");
	      print_overlay( @$.first_column, @$.last_column, "curlyContent1_Labelled");	  

	      $$ = NULL;   //Tree = NULL;
	    }

curlyContentA_Labelled:
            curlyContentB_Labelled SEMI postDocument
            {
	      // printf("curlA_Labelled\n");
		print_overlay( @$.first_column, @$.last_column, "curlyContentA_Labelled");
	      $$ = newTempNode(TK);
	      $$->child[0] = $1;
	      $$->child[1] = $3;
	    }
                                        
                                         
curlyContentB_Labelled:
            preDocument labelled postDocument
            {
	      // printf("curlB_Labelled\n");
		print_overlay( @$.first_column, @$.last_column, "curlyContentB_Labelled");
	      
	      if( $1 == NULL && $2 == NULL )
		$$ = $3;

	      else if ($1 == NULL && $3 == NULL)
		$$ = $2;

	      else if ($2 == NULL && $3 == NULL)
		$$ = $1;
	      
	      else
	      {
		$$ = newTempNode(TK);
		$$->child[0] = $1;
		$$->child[1] = $2;
		//printTree($2);
		$$->child[2] = $3;
	      }
	    }



%%

#include <string.h>


int main(int argc, char * argv[] )
{

    char* programfile; //the sourse file to compile in pascal


    // check to see if the user input 2 argument one the program and one the
    // file to compiled
    if(argc != 2)
    {
        // log if the format given is not corredt and exits
        printf("two arguments are needed usage: %s <filename.pas>\n",argv[0]);
        exit(1);
    }

    programfile = argv[1];

    // opnes the source for use in the compiler then checks to see where the
    // file exist or not if does not log problem and exits
    yyin = fopen(programfile, "r");
    if(yyin == NULL)
    {
        printf("compile file not found logout\n");
        exit(1);
    }
    yyparse();

    printTree(savedTree);

    printf("PARINSG COMPLETE\n");
    fclose(yyin);
    return 0;
}



