/* ALL the global variables and functions declaration */


#ifndef _GLOBALS_H_

#define _GLOBALS_H_



#include <stdio.h>

#include <stdlib.h>

#include <ctype.h>

#include <string.h>

#ifndef FALSE

#define FALSE 0

#endif

#ifndef TRUE

#define TRUE 1

#endif



/* MAXRESERVED = the number of reserved words in ALDOR*/

#define MAXRESERVED 57


typedef int TokenType;


extern int line_num; /* source line number for listing */



/**************************************************/

/***********   Syntax tree for parsing ************/

/**************************************************/



typedef enum {DeclK, ExpK, StmtK, InfixK, EncK, BlockK, TempK} NodeKind;




typedef enum {
               MacroK, ExtendK, LocalK, FluidK, DefaultK, 
	       DefineK, FixK, InlineK, ImportK, ExportK,

	       FreeK
} DeclKind;


typedef enum { 
               IfK, TryK, ExitK, RepeatK, SelectK,
	       DoK, DelayK, GenerateK, AssertK, IterateK,
	       BreakK, ReturnK, YieldK, ExceptK, GotoK, 
	       NeverK, ForK, WhileK,

	       WhereK, ToK, AlwaysK, FinallyK, AndK, OrK,
	       HasK, PretendK, WithK, AddK, NotK,
	       IsntK, IsK, CaseK, ByK, ModK, QuoK, RemK
} StmtKind;

typedef enum {InfixedK} InfixKind;
typedef enum {OpK, ConstK, IdK, DocK} ExpKind;
typedef enum {ParenedK, BracketedK, QuotedIdsK} EncKind;
typedef enum {PiledK, CurlyK, Curly1K, StartK, DocBlockK} BlockKind;
typedef enum {TK} TempKind;


typedef enum {Integer, Float, String, Default} ExpType;

#define MAXCHILDREN 3 // the maximum amount of children nodes off 1 node



typedef struct treeNode
{
  struct treeNode * child[MAXCHILDREN];

  struct treeNode * sibling;

  int lineno;
  int start_position;
  int end_position;
  char* token_type;

  NodeKind nodekind;

  union { 
    DeclKind decl;
    StmtKind stmt;
    ExpKind exp;
    EncKind enc;
    BlockKind block;
    InfixKind infixed;
    TempKind temp;
  }kind;

  union {
    TokenType op;
    char* name;
    ExpType type;
  }attr;

  ExpType type; /* for type checking of exps */

} TreeNode;


#endif


