// Print function for the tokens
void printToken( TokenType token, const char* tokenString)
{
  switch (token)
  {
    // RESERVED WORDS 
    case ADD:
    case ALWAYS:
    case AND:
    case ASSERT:
    case BREAK:
    case BUT:
    case CATCH:
    case DEFAULT:
    case DEFINE:
    case DELAY:
    case DO:
    case ELSE:
    case EXCEPT:
    case EXPORT:
    case EXTEND:
    case FIX:
    case FLUID:
    case FOR:
    case FREE:
    case FROM :
    case GENERATE:
    case GOTO:
    case HAS:
    case IF:
    case IMPORT:
    case IN:
    case INLINE:
    case IS:
    case ISNT:
    case ITERATE:
    case LET:
    case LOCAL:
    case MACRO:
    case NEVER:
    case NOT:
    case OF:
    case OR:
    case PRETEND:
    case REF:
    case REPEAT:
    case RETURN:
    case RULE:
    case SELECT:
    case THEN:
//  case THROW:
    case TO:
    case TRY:
    case WHERE:
    case WHILE:
    case WITH:
    case YIELD:
    case BY:
    case CASE:
    case FINALLY:
    case MOD:
    case QUO:
    case REM:
      printf("RESERVED WORD: %s\n",tokenString);
      break;

    // SYMBOLS - DEFINEABLE
    case SETMINUS: printf("\\\n"); break;
    case POUND: printf("#\n"); break;
    case TIMES: printf("*\n"); break;
    case PLUS: printf("+\n"); break;
    case MINUS: printf("-\n"); break;
    case DIVIDE: printf("/\n"); break;
    case LESSTHAN: printf("<\n"); break;
    case EQUALS: printf("=\n"); break;
    case GREATERTHAN: printf(">\n"); break;
    case ATSIGN: printf("@\n"); break;
    case TILDE: printf("~\n"); break;
    case CARET: printf("^\n"); break;
    case VERY_LESS: printf("<<\n"); break;
    case VERY_GREATER: printf(">>\n"); break;
    case LESSEQ: printf("<=\n"); break;
    case GREATEREQ: printf(">=\n"); break;
    case GETS: printf("<-\n"); break;
    case TO_SYMBOL: printf("->\n"); break;
    case JOIN: printf("\\/\n"); break;
    case MEET: printf("/\\\n"); break;
    case STARSTAR: printf("**\n"); break;
    case DOTDOT: printf("..\n"); break;
    case UNEQUALS: printf("-=\n"); break;
    case CARETEQUALS: printf("^=\n"); break;

    // OPERATORS - RESERVED
    case DOLLAR: printf("$\n"); break;
    case SQUOTE: printf("'\n"); break;
    case LPAREN: printf("(\n"); break;
    case RPAREN: printf(")\n"); break;
    case FUN_ARROW: printf("+->\n"); break;
    case COMMA: printf(",\n"); break;
    case COLON: printf(":\n"); break;
    case IS_ASSIGNED: printf(":=\n"); break;
    case SEMI: printf(";\n"); break;
    case MACRO_IS: printf("==>\n"); break;
    case EXIT: printf("=>\n"); break;
    case LBRACK: printf("[\n"); break;
    case RBRACK: printf("]\n"); break;
    case LBRACE: printf("{\n"); break;
    case RBRACE: printf("}\n"); break;
    case VERY_EQUAL: printf("==\n"); break;
    case BAR: printf("|\n"); break;

    // OPERATORS - FUTURE
    case LLBRACK: printf("[|\n"); break;
    case LLBRACE: printf("{|\n"); break;
    case LLPAREN: printf("(|\n"); break;
    case RRBRACK: printf("|]\n"); break;
    case RRBRACE: printf("|}\n"); break;
    case RRPAREN: printf("|)\n"); break;
    case GRAVE: printf("`\n"); break;
    case AMPERSAND: printf("&\n"); break;
    case OROR: printf("||\n"); break;

    // OTHERS
    case INT: printf("INTEGER: val= %s\n", tokenString); break;
    case ID: printf("IDENTIFIER: val= %s\n", tokenString); break;
    case FLOAT: printf("FLOAT: val= %s\n",tokenString); break;
    case PREDOC: printf("PREODCUMENT: val= %s\n", tokenString); break;
    case POSTDOC: printf("POSTDOCUMENT: val= %s\n", tokenString); break;
    case STRING: printf("STRING: val= %s\n",tokenString); break;

   
    case DCOLON: printf("::\n"); break;
    case COLON_STAR: printf(":*\n"); break;
    case DOT: printf(".\n"); break;
    case PLUSMINUS: printf("+-\n"); break;
    

    default: /* should never happen */
      printf("UNKNOWN TOKEN: %d\n",token);
      break;
  }

}




// newNode() create a new Node with all the initialization
TreeNode * newNode()
{
  TreeNode * t = (TreeNode *) malloc(sizeof(TreeNode));

  if (t==NULL)
    printf("Out of memory error at line %d\n",line_num);
  
  else{
    for (int i = 0; i < MAXCHILDREN; i++) 
      t->child[i] = NULL;

    t->sibling = NULL;
    t->lineno = line_num;
  }
  
  return t;
}

// newDeclNode(DeclKind) create new Declaration Node
TreeNode * newDeclNode(DeclKind kind)
{
  TreeNode * t = newNode();

  if( t != NULL)
  {
    t->nodekind = DeclK;
    t->kind.decl = kind;
  }

  return t;
}


// newStmtNode(StmtKind) create new Statement Node
TreeNode * newStmtNode(StmtKind kind)
{
  TreeNode * t = newNode();

  if( t != NULL)
  {
    t->nodekind = StmtK;
    t->kind.stmt = kind;
  }

  return t;
}


// newInfixNode(InfixKind) create new Infixed Node
TreeNode * newInfixNode(InfixKind kind)
{
  TreeNode * t = newNode();

  if( t != NULL)
  {
    t->nodekind = InfixK;
    t->kind.infixed = kind;
  }

  return t;
}


// newExpNode(ExpKind) create new Expression Node
TreeNode * newExpNode(ExpKind kind)
{
  TreeNode * t = newNode();

  if( t != NULL)
  {
    t->nodekind = ExpK;
    t->kind.exp = kind;
  }

  return t;
}

// newEncNode(EncKind) create new Enclosure Node
TreeNode * newEncNode(EncKind kind)
{
  TreeNode * t = newNode();

  if( t != NULL)
  {
    t->nodekind = EncK;
    t->kind.enc = kind;
  }

  return t;
}

// newBlockNode(BlockKind) create new Block Node
TreeNode * newBlockNode(BlockKind kind)
{
  TreeNode * t = newNode();

  if( t != NULL)
  {
    t->nodekind = BlockK;
    t->kind.block = kind;
  }

  return t;
}

// newTempNode(TempKind) create new Temp Node
TreeNode * newTempNode(TempKind kind)
{
  TreeNode * t = newNode();

  if( t != NULL)
  {
    t->nodekind = TempK;
    t->kind.temp = kind;
  }

  return t;
}



// newName(char* name) create a new node with the name "name"
TreeNode * newName(char* name)
{
  TreeNode * t = newNode();

  if( t != NULL)
    t->attr.name = name;
    
  return t;
}

// copyString allocates and makes a new copy of an existing string
char * copyString( char * s)
{ 
  int n;
  char * t;

  if (s == NULL) 
    return NULL;

  n = strlen(s) + 1;
  t = (char *) malloc(n);

  if (t==NULL)
    printf("Out of memory error at line %d\n",line_num);

  else 
    strcpy(t,s);

  return t;
}




// Variable indentno is used by printTree to
// store current number of spaces to indent
static int indentno = 0;
static bool normal = true;
int token_pos = 0;

/* macros to increase/decrease indentation */
#define INDENT indentno += 2
#define UNINDENT indentno -= 2


/* printSpaces indents by printing spaces */
static void printSpaces()
{ 
  int i;

  for (i = 0; i < indentno; i++)
    printf(" ");
}


// printTree( TreeNode* ) print the parse tree
void printTree( TreeNode * tree, char* programfile)
{
  while (tree != NULL) 
  {
    if( tree->nodekind != TempK )
    {
      printf( "(setq overlay-%d (make-overlay %d %d (get-buffer \"%s\")))", token_pos, tree->start_position, tree->end_position, programfile);
      printf( "(overlay-put overlay-%d 'aldor-syntax' %s)", token_pos, 
	      tree->token_type);
      ++token_pos;
    }

    for (int i = 0; i < MAXCHILDREN; ++i)
      printTree(tree->child[i], programfile);

    tree = tree->sibling;
  }
}


